# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions for parsing the commandline."""

from Common import *

from Config import config

import getopt
import os.path
import sys

def parse_and_start():
    """Parses the commandline arguments given to mattricks.py."""

    usage = _("""Usage: %s [general options] command [command-specific options]

Options:

  -h    --help          Show this help message and exit.
  -H    --help-command  Show help on options for the specified command.

        --version       Print version info and exit.

Commands:

  gui          Start up the gui. [The default]
  fetch        Fetch an hrf/xml-file and exit.
  listchanges  List the changes between two hrf-files and exit.
  maketeam     Let the computer generate a team from hrf and exit.
  table        Print a table from a fixture-xml and exit.
  import       Import xml files into the database.
  players      Display statistics on players performances in games.
""") % sys.argv[0]

    try:
        (options, args) = getopt.getopt(sys.argv[1:], "hH",
                                        ["help",
                                         "help-command",
                                         #"datadir=",
                                         #"localedir=",
                                         "version"])
    except getopt.GetoptError, e:
        sys.stdout.write(str(e) + "\n\n")
        sys.stdout.write(usage)
        sys.exit(2)

    help_command = False
    for o, a in options:
        if o in ["-h", "--help"]:
            sys.stdout.write(usage)
            sys.exit()
        elif o in ["-H", "--help-command"]:
            help_command = True
        elif o in ["--version"]:
            import Version

            version_text = "Mattricks " + Version.version + "\n\n" + \
                           _("""Copyright (C) 2003-2004 Mattricks Team
This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
""")

            sys.stdout.write(version_text)
            sys.exit()

    all_commands = {
        "gui": parse_cmd_gui,
        "fetch": parse_cmd_fetch,
        "listchanges": parse_cmd_listchanges,
        "maketeam": parse_cmd_maketeam,
        "table": parse_cmd_table,
        "import": parse_cmd_import,
        "players": parse_cmd_players,
        }

    command = "gui"

    if len(args) > 0:
        command = args[0]
        args = args[1:]

    if not all_commands.has_key(command):
        sys.stdout.write(_("Command %s not recognized.") % command + "\n\n")
        sys.stdout.write(usage)
        sys.exit(2)

    all_commands[command](args, help_command)

def parse_cmd_gui(args, help_command):
    """Parse the arguments for command 'gui'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    gui_usage = _("Usage: %s gui") % sys.argv[0] + "\n"

    if help_command:
        sys.stdout.write(gui_usage)
        sys.exit()

    try:
        import Main
        Main.main(*args)
    except ImportError, e:
        sys.stdout.write(_("Unable to import the Mattricks package.") + "\n\n")
        sys.stdout.write(str(e) + "\n")

    sys.exit()

def parse_cmd_fetch(args, help_command):
    """Parse the arguments for command 'fetch'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    fetch_usage = _("""Usage: %s fetch [options]

Options:

  -u USER    --user=USER         Specify the hattrick user (if
                                 omitted it is asked for).
  -p PASS    --pass=PASS         Specify the hattrick password (if
                                 omitted it is asked for).
             --passwordfile=FILE A file contaning only loginname and
                                 password.
""") % sys.argv[0]

    if help_command:
        sys.stdout.write(fetch_usage)
        sys.exit()

    try:
        (options, args) = getopt.getopt(args, "p:u:",
                                        ["pass=",
                                         "passwordfile=",
                                         "user="])
    except getopt.GetoptError, e:
        sys.stdout.write(str(e) + "\n\n")
        sys.stdout.write(fetch_usage)
        sys.exit(2)


    import string

    loginname = None
    password = None

    for o, a in options:
        if o in ["-u", "--user"]:
            loginname = a
        elif o in ["-p", "--pass"]:
            password = a
        elif o in ["--passwordfile"]:
            try:
                passwordfile = open(a, 'r').read()
                (loginname, password) = string.split(passwordfile, maxsplit = 1)
                loginname = loginname.strip()
                password = password.strip()
            except Exception, e:
                sys.stdout.write(_("Couldn't get loginname and password") + ":\n" +
                                 str(e) + "\n\n")
                sys.stdout.write(fetch_usage)
                sys.exit(2)

    if not loginname:
        sys.stdout.write(_("Hattrick login name") + ": ")
        loginname = sys.stdin.readline().strip()

    if not password:
        import getpass
        password = getpass.getpass(_("Hattrick password") + ": ")

    import Fetch, CommonData, DialogSet
    from Database import databases

    DialogSet.progress_dialog_maker_func = None

    databases.sets(None, True, {"Mattricks.loginname": loginname,
                                "Mattricks.password": password})

    try:
        res = CommonData.download_common_data()
    except Fetch.FetchError, e:
        sys.stderr.write(_("Error") + ": " + str(e) + "\n")
        if e.extra_info:
            sys.stderr.write("\n" + e.extra_info + "\n")

def parse_cmd_listchanges(args, help_command):
    """Parse the arguments for command 'listchanges'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    import string
    import HRFData, Listchanges

    listchanges_usage = _("""Usage: %s listchanges [options] first_hrf second_hrf

Options:

  -f LIST    --filter=LIST   A comma separated list of filters.

Filters: %s
""") % (sys.argv[0], string.join(Listchanges.filters.keys(), ", "))

    if help_command:
        sys.stdout.write(listchanges_usage)
        sys.exit()

    try:
        (options, args) = getopt.getopt(args, "f:", ["filter="])
    except getopt.GetoptError, e:
        sys.stdout.write(str(e) + "\n\n")
        sys.stdout.write(listchanges_usage)
        sys.exit(2)

    filter_list = []
    for o, a in options:
        if o in ["-f", "--filter"]:
            filter_list = unique(filter_list + string.split(a, ","))

    for f in filter_list:
        if f not in Listchanges.filters.keys():
            sys.stdout.write(_("Filter type %s does not exist.") % f + "\n\n")
            sys.stdout.write(listchanges_usage)
            sys.exit(2)

    if len(args) != 2:
        sys.stdout.write(_("Wrong number of hrf files.") + "\n\n")
        sys.stdout.write(listchanges_usage)
        sys.exit(2)

    h1 = HRFData.HRFData(args[0])
    h2 = HRFData.HRFData(args[1])

    sys.stdout.write(Listchanges.Listchanges(h1, h2).get_changes_text(filter_list) + "\n")

def parse_cmd_maketeam(args, help_command):
    """Parse the arguments for command 'maketeam'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    maketeam_usage = _("Usage: %s maketeam hrffile") % sys.argv[0] + "\n"

    if help_command:
        sys.stdout.write(maketeam_usage)
        sys.exit()

    if len(args) != 1:
        sys.stdout.write(_("Wrong number of hrf files.") + "\n\n")
        sys.stdout.write(maketeam_usage)
        sys.exit(2)

    import string
    import TeamSelector, HRFData, Terms

    h1 = HRFData.HRFData(args[0])
    ts = TeamSelector.TeamSelector(h1)
    pt = ts.get_player_table()
    lineup = ts.select_team()

    for f, plist in pt.items():
        print
        print f.capitalize() + ":"
        print
        for val, pid in plist[:8]:
            print "%5.2f %s" % (val, h1.get_player(pid).name)

    print "\n" + _("Team") + ":\n"
    for descr, cpos, pos in TeamSelector.TeamSelector.lineup_info:
        print "%s %s %d %.2f %s" % (
            string.ljust(descr, 13),
            string.ljust(Terms.individual_order[lineup[cpos][4]] + ":", 16),
            lineup[cpos][2],
            lineup[cpos][1],
            h1.get_player(lineup[cpos][0], True).name)

    print "\n%s   %.2f" % (string.ljust(_("Team value") + ":", 30),
                           ts.de.teamval(lineup))

def parse_cmd_table(args, help_command):
    """Parse the arguments for command 'table'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    table_usage = _("""Usage: %s table [options]

Options:

  -l LEAGUE  --league-id=LEAGUE  The league id of the table.
  -s SEASON  --season=SEASON     Which season to use.
  -r ROUND   --round=ROUND       Print the table for this ROUND.
""") % sys.argv[0]

    if help_command:
        sys.stdout.write(table_usage)
        sys.exit()

    try:
        (options, args) = getopt.getopt(args, "l:r:s:", ["league-id=",
                                                         "round=",
                                                         "season="])
    except getopt.GetoptError, e:
        sys.stdout.write(str(e) + "\n\n")
        sys.stdout.write(table_usage)
        sys.exit(2)

    league_id = -1
    season = -1
    round = 14
    for o, a in options:
        if o in ["-l", "--league-id"]:
            try:
                league_id = int(a)
            except ValueError, e:
                sys.stdout.write(_("Id %s is not a valid league id.") % a + "\n\n")
                sys.stdout.write(table_usage)
                sys.exit(2)
        if o in ["-s", "--season"]:
            try:
                season = int(a)
            except ValueError, e:
                sys.stdout.write(_("Value %s is not a valid season.") % a + "\n\n")
                sys.stdout.write(table_usage)
                sys.exit(2)
        if o in ["-r", "--round"]:
            try:
                round = int(a)
                if not (0 < round <= 14):
                    raise ValueError
            except ValueError, e:
                sys.stdout.write(_("Round %s is not a valid round.") % a + "\n\n")
                sys.stdout.write(table_usage)
                sys.exit(2)

    import Fixtures

    Fixtures.Fixtures(league_id, season).write_table_text(sys.stdout, round)

def parse_cmd_import(args, help_command):
    """Parse the arguments for command 'import'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    import_usage = _("Usage: %s import xml-files") % sys.argv[0] + "\n"

    maketeam_usage = _("Usage: %s maketeam hrffile") % sys.argv[0] + "\n"

    if help_command:
        sys.stdout.write(import_usage)
        sys.exit()

    if len(args) < 1:
        sys.stdout.write(_("Needs at least one xml file.") + "\n\n")
        sys.stdout.write(import_usage)
        sys.exit(2)

    import Fetch

    for arg in args:
        try:
            Fetch.xml_import(arg)
        except FetchError, err:
            sys.stderr.write(_("Error") + ": " + str(e) + "\n")
            if e.extra_info:
                sys.stderr.write("\n" + e.extra_info + "\n")

def parse_cmd_players(args, help_command):
    """Parse the arguments for command 'players'.

     - args -- The arguments given after the command.
     - help_command -- If we should print the help string."""

    import os, re
    import string, time
    import HRFData, MatchInfo, Terms, Fixtures, CommonData, XMLParse

    player_usage = _("""Usage: %s players [options]
Options:

  -s SEASON  --season=SEASON     Which season to use.
  -t TEAM    --team-id=TEAM      Print the info for this TEAM.
""") % (sys.argv[0])

    if help_command:
        sys.stdout.write(player_usage)
        sys.exit()

    try:
        (options, args) = getopt.getopt(args, "t:s:", ["team-id=",
                                                       "season="])
    except getopt.GetoptError, e:
        sys.stdout.write(str(e) + "\n\n")
        sys.stdout.write(player_usage)
        sys.exit(2)

    league_id = None
    team_id = None
    season = None

    for o, a in options:
        if o in ["-t", "--team-id"]:
            try:
                team_id = int(a)
            except ValueError, e:
                sys.stdout.write(_("Id %s is not a valid team id.") % a + "\n\n")
                sys.stdout.write(player_usage)
                sys.exit(2)
    for o, a in options:
        if o in ["-s", "--season"]:
            try:
                season = int(a)
            except ValueError, e:
                sys.stdout.write(_("%s is not a valid season.") % a + "\n\n")
                sys.stdout.write(player_usage)
                sys.exit(2)

    # Get the default team_id
    if team_id is None:
        team_id = CommonData.get_current_teamid()
        sys.stdout.write(_("Setting default %s: %d.") % (_("team"), team_id) + "\n")

    # Use the team_id to get the league_id and season, unless specified
    data = XMLParse.get(["team details", team_id],
                        ["Team.League.LeagueID",
                        "Team.LeagueLevelUnit.LeagueLevelUnitID"])

    league_id = data.get("Team.LeagueLevelUnit.LeagueLevelUnitID")

    if season is None:
        lid = data.get("Team.League.LeagueID")
        if lid is not None:
            season = CommonData.get_league_season(lid)
            sys.stdout.write(_("Setting default %s: %d.") % (_("season"), season) + "\n")

    if league_id is None or team_id is None or season is None:
        sys.stdout.write(_("Unable to find specified team, league or season.") + "\n\n")
        sys.stdout.write(player_usage)
        sys.exit(2)

    players = {}

    fixture = Fixtures.Fixtures(league_id, season)
    sys.stdout.write(_("Team %d %s(%d) Season %d") % 
                     (team_id, fixture.league_name, league_id, fixture.season) + "\n")

    # Loop thru every round this season. Should gather data from matchlist instead
    for round in range(0, 14):
        for match_id, match_info in fixture.matches[round + 1].iteritems():
            if not match_info[3]:
                continue

            # Check if Im one of the teams
            if match_info[1][0] == team_id or match_info[2][0] == team_id:
                # Print each match as we see them
                match_lineup = MatchInfo.MatchLineup(match_id, team_id)
                sys.stdout.write("    %s: %s - %s %.1f\n" % 
                                (time.strftime("%Y-%m-%d", time.gmtime(match_lineup.match_date)),
                                 match_info[1][1], match_info[2][1],
                                 match_lineup.total_rating))

                # Loop thru every position
                for i in range(1,22):
                    (id, name, rating, position, behaviour) = match_lineup.lineup[i]

                    if id is None or id is 0 or rating is None:
                        continue

                    # Save the info. The players var is a map where each id points at
                    # a array of arrays. Each top array is one match-performance and
                    # it contains date, rating, position and behaviour """
                    if players.has_key(id):
                        players[id]["matches"] += [[match_lineup.match_date, rating,
                                                    position, behaviour]]
                    else:
                        players[id] = {"name":name, "matches":[[match_lineup.match_date, rating,
                                                                position, behaviour]]}

    sys.stdout.write("\n\n")
    # Repeat thru the players we have found
    for playerid, playerinfo in players.iteritems():
        sys.stdout.write(playerinfo["name"] + " (" + str(playerid) + "):\n")
        for match in playerinfo["matches"]:
            match_date, rating, position, behaviour = match
            sys.stdout.write("    " + time.strftime("%Y-%m-%d", time.gmtime(match_date)) +
                             ": " + str(rating) + "*")
            if position is None:
                sys.stdout.write(" (" + _("Unknown") + ")\n")
            else:
                sys.stdout.write(" at " + Terms.team_role[position] + ", " +
                                 Terms.individual_order[behaviour] + "\n")
        sys.stdout.write("\n")
