# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Common functions used everywhere in Mattricks."""

import __builtin__
import string
import time
import re
import sys
import math
import operator
import os

import Version
import I18n

if not __builtin__.__dict__.has_key("_"):
    __builtin__.__dict__['_'] = I18n.no_translation
    __builtin__.__dict__['N_'] = I18n.no_translation
    __builtin__.__dict__['ngettext'] = I18n.no_translation_plural

def version_str():
    """Returns a list of strings with version information."""
    return (_("This is Mattricks %s") % Version.version,
            _("It runs under python %s on %s") % (sys.version, sys.platform))

def string2time(tstring):
    """Takes a date string in standard HT time format and converts it to a
    unix timestamp."""

    r = re.compile("^([0-9]{1,4})-([0-9]{1,2})-([0-9]{1,2})( ([0-9]{2}):([0-9]{2}):([0-9]{2}))?$")
    m = r.match(tstring.strip())
    if not m:
        return None

    if m.group(4) is None:
        res = m.group(1,2,3) + (0, 0, 0)
    else:
        res = m.group(1,2,3,5,6,7)
    return time.mktime(tuple([int(x) for x in res]) + (0, 0, -1))

def name2surname(name_string):
    """Make a surname string from a name string.

    Assumes that the person just have one first name.
    'Adam Adamsson' => 'Adamsson, Adam'"""

    name = string.split(name_string, " ", 1)
    name.reverse()
    return string.join(name, ", ")

def unique(the_list):
    """Removes all duplicates in the list."""
    res = []
    for l in the_list:
        if not l in res:
            res.append(l)
    return res

def intersection(seq1, seq2):
    """The intersection of two sequences."""

    result = []
    for element in seq1:
        if element in seq2 and element not in result:
            result.append(element)

    for element in seq2:
        if element in seq1 and element not in result:
            result.append(element)

    return result

def mktimestring(ctime = None):
    """Make a time string from a unix timestamp.

    Uses current time by default."""

    if ctime == None:
        ctime = time.time()

    return time.strftime("%A, %d %b %Y %H:%M:%S", time.localtime(ctime)).capitalize()

def mkdatestring(ctime = None):
    """Make a date (with just the date) string from a unix timestamp.

    Uses current time by default."""

    if ctime == None:
        ctime = time.time()

    return time.strftime("%A, %d %b %Y", time.localtime(ctime)).capitalize()

def mkHTtimestring(ctime = None):
    """Make a HT time string from a unix timestamp.

    Uses current time by default."""

    if ctime == None:
        ctime = time.time()

    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(ctime)).capitalize()

def time2ymd(ctime = None):
    """Takes a unix timestamp and returns (year, month, date) in local time.

    Uses current time by default."""

    if ctime == None:
        ctime = time.time()

    return time.localtime(ctime)[:3]

def divmod(dividend, *divisors):
    """Multiple divisor version of divmod. This function is got from a PEP
    by Thomas Bellman."""
    modulos = ()
    q = dividend
    while divisors:
        q,r = q.__divmod__(divisors[-1])
        modulos = (r,) + modulos
        divisors = divisors[:-1]
    return (q,) + modulos

def sgn(value):
    """Get the sign of the value."""
    if value == 0.0:
        return value
    return value/abs(value)

def round_prec(value, precision = 1):
    """Like round, but specify a precision instead of an absolute decimal point."""
    return round(value, -int(math.floor(math.log10(abs(value)))) + precision - 1)

def htdata_text(htdata):
    """Make a text with info about htdata."""
    if htdata == (None, None):
        return "\n\n"
    text = _("Date") + ": %s\n" % mktimestring(htdata[1].get_section("basics").date) + \
           _("File") + ": %s\n" % htdata[0] + \
           _("Team") + ": %s" % htdata[1].get_section("basics").teamName
    return text

class dict2func:
    def __init__(self, d):
        self.d = d

    def __call__(self, key):
        return self.d[key]


def add_currency_to_money(amount, currency, is_prefix, always_sign = False):
    """Make a money string."""
    if always_sign:
        sign = {True: "-", False: "+"}[bool(amount < 0)]
    else:
        sign = {True: "-", False: ""}[bool(amount < 0)]
    amount = string.join(split_equal(str(abs(amount)), 3, True), " ")
    if is_prefix:
        return sign + currency + amount
    else:
        return sign + amount + " " + currency

def signedstr(value):
    """Return a string with explicit sign for numbers."""
    if value > 0:
        text = ("+" + str(value))
    else:
        text = str(value)
    return text

def split_equal(mlist, size, reverse = False):
    """Split a string or list in equal places."""
    res = []
    pos = 0
    while pos < len(mlist):
        if reverse:
            if pos == 0:
                res.insert(0,mlist[-(pos+size):])
            else:
                res.insert(0,mlist[-(pos+size):-pos])
        else:
            res.append(mlist[pos:pos+size])
        pos += size
    return res

def string2bool(x):
    """Returns True if the string is True, Yes or 1, otherwise False."""
    if x.lower() in ["true", "yes", "1"]:
        return True
    return False

def any_in(a,b):
    """True if any of the elements in the sequence a also exists in sequence b."""
    return reduce(operator.or_, [(i in b) for i in a])

def double_in(a,b):
    """True if the sequence a is part of any of the sequences in the sequence of sequences b."""
    return reduce(operator.or_, [(a in i) for i in b])

def any_begins_with(a,b):
    """True if any of the sequences in the sequence b begins with sequence a."""
    l = len(a)
    return reduce(operator.or_, [(a == i[:l]) for i in b])

def normal_distribution(m = 0.0, sigma = 1.0):
    """Returns the density function of the normal distribution."""
    if sigma == 0.0:
        return (lambda x: 1.0)
    t = 1.0/(sigma*math.sqrt(2.0*math.pi))
    d = -1.0/(2.0*sigma*sigma)
    return (lambda x, m=m, t=t, d=d: t*math.exp(pow((float(x)-m),2)*d))

def binomial_coefficient(n, k):
    """Calculates the binomial coefficient n over k."""
    if k == 0 or k == n:
        return 1

    x = reduce(operator.mul, range(n-k+1, n+1))
    y = reduce(operator.mul, range(1, k+1))

    return x // y

def binomial_distribution(n, p):
    """Returns the probability function of the binomial distribution."""
    return (lambda k, n=n, p=p: binomial_coefficient(n,k)*pow(p,k)*pow(1.0-p,n-k))
