# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Common functions for getting various kinds of data."""

from Common import *

import Countries

def add_currency(amount_of_money, always_sign = False):
    """Create a currency string from an amount of money
    using current teams league currency."""

    lid = get_country_league_id()
    (htname, name, currency, is_prefix, exchangerate, flag) = Countries.country_info[lid]
    amount = int(float(amount_of_money) / float(exchangerate))
    return add_currency_to_money(amount, currency, is_prefix, always_sign)

import Database
import XMLParse

def get_league_season(league_id):
    """Returns the current season of a league.

     - league_id -- The league id of the league to get season from."""

    data = XMLParse.get(["world details"], ["LeagueList.League.xml_list",
                                            "LeagueList.League.xml_attrs.Index"])

    lindeces = data.get("LeagueList.League.xml_list")
    if lindeces is None:
        lindeces = data.get("LeagueList.League.xml_attrs.Index")
        if lindeces is None:
            return
        else:
            lindeces = [str(lindeces)]
    else:
        lindeces = ["+" + x for x in lindeces.split(",")]

    paths = []
    for index in lindeces:
        paths.append("LeagueList.League%s.LeagueID" % index)
        paths.append("LeagueList.League%s.Season" % index)

    data = XMLParse.get(["world details"], paths)

    for index in lindeces:
        if league_id == data.get("LeagueList.League%s.LeagueID" % index):
            return data.get("LeagueList.League%s.Season" % index)

    return None

def get_current_teamid(ret_userid = False):
    """Returns the current teamid or userid.

    If necessary, downloads a new team details xml to determine
    teamid and userid.

     - ret_userid -- if True, returns the userid instead of the teamid.
    """

    data = Database.databases.gets(None, False, ["Mattricks.current_teamid",
                                                 "Mattricks.current_userid"])
    teamid = data.get("Mattricks.current_teamid")
    userid = data.get("Mattricks.current_userid")

    if teamid is None or userid is None or len(teamid) == 0 or len(userid) == 0:
        data = XMLParse.get(["team details", -1], ["Team.TeamID", "User.UserID"])

        teamid = data.get("Team.TeamID")
        userid = data.get("User.UserID")

        if teamid is not None and userid is not None:
            Database.databases.sets(None, False, {"Mattricks.current_teamid": str(teamid),
                                                  "Mattricks.current_userid": str(userid)})
    else:
        teamid = int(teamid)
        userid = int(userid)

    if ret_userid:
        return userid
    return teamid

def is_supporter():
    """Returns true if the current user is a supporter"""
    data = Database.databases.gets(None, False, ["Mattricks.current_is_supporter"])
    supporter = data.get("Mattricks.current_is_supporter")

    if supporter is None or len(supporter) == 0:
        supporter = XMLParse.get(["team details", -1], ["User.HasSupporter"])
        if supporter is not None:
            Database.databases.sets(None, False, {"Mattricks.current_is_supporter": str(supporter)})
    else:
        supporter = string2bool(supporter)

    return supporter

country_league_id_cache = {}

def get_country_league_id(teamid = None):
    """Returns the league id of the country for the given team.

    This function caches the retrieved info for best speed.

     - teamid -- the teamid to get country info for,
                 if None uses current team."""

    global country_league_id_cache

    if country_league_id_cache.has_key(teamid):
        return country_league_id_cache[teamid]

    tid = teamid

    while tid is None:
        tid = get_current_teamid()

    data = XMLParse.get(["team details", tid], ["Team.League.LeagueID"])
    if data is not None:
        country_league_id_cache[teamid] = data
    return data

def get_xml_list_indices(path, data, xml_attr = "Index"):
    """Get all indices of a particular path.

     - path -- The path to the list of the indeces.
     - data -- The data to get from.
     - xml_attr -- The index form in the data to use."""

    indices = data.get(path + ".xml_list", "").split(",")
    if len(indices) == 1 and indices[0] == "":
        m = data.get(path + ".xml_attrs." + xml_attr)
        if m is None:
            indices = []
    else:
        indices = ["+" + x for x in indices]

    return indices

def download_common_data():
    teamid = get_current_teamid()

    data = XMLParse.get(["world details"], ["FileName"], force_update = True)

    if data is not None:
        data = XMLParse.get(["team details", teamid], ["FileName"], force_update = True)

    if data is not None:
        data = XMLParse.get(["all players", teamid, time.localtime(time.time())[0:4]],
                            ["FileName"], force_update = True)

    if data is not None:
        data = XMLParse.get(["economy", time.localtime(time.time())[0:4]],
                            ["FileName"], force_update = True)

    if data is not None:
        data = XMLParse.get(["training", time.localtime(time.time())[0:4]],
                            ["FileName"], force_update = True)

    if data is not None:
        data = XMLParse.get(["arena details", time.localtime(time.time())[0:4]],
                            ["FileName"], force_update = True)

    if data is not None:
        data = XMLParse.get(["club", time.localtime(time.time())[0:4]],
                            ["FileName"], force_update = True)

    return (data is not None)

class HTdate:
    def __init__(self):
        self.__got_base_date = False
        self.__base_time = None

    def from_local(self, t = None):
        if not self.__got_base_date:
            self.__get_conversion_info()

        if self.__base_time is None:
            return (1, 1)

        if t is None:
            t = time.time()

        difftime = t - self.__base_time
        (season, day, rest) = divmod(difftime, 7*16, 86400)
        return (int(season), int(day + 1))

    def to_unix_time(self, htdate):
        if not self.__got_base_date:
            self.__get_conversion_info()

        if self.__base_time is None:
            return 0.0

        return self.__base_time + ((((htdate[0] * 16) * 7) + htdate[1] - 1) * 86400)

    def __get_conversion_info(self):
        self.__got_base_date = True

        teamid = get_current_teamid()
        league_id = get_country_league_id()

        data = XMLParse.get(["world details"])

        fetched_date = data.get("FetchedDate")
        if fetched_date is None:
            self.__base_time = None
            return

        l_indices = get_xml_list_indices("LeagueList.League", data)
        for lindex in l_indices:
            c_lid = data.get("LeagueList.League%s.LeagueID" % lindex)

            if c_lid == league_id:
                season = data.get("LeagueList.League%s.Season" % lindex)
                match_round = data.get("LeagueList.League%s.MatchRound" % lindex)
                break

        flocal = time.localtime(fetched_date)
        cbase = flocal[0:2] + (flocal.tm_mday - flocal.tm_wday,)

        self.__base_time = time.mktime(cbase + (0,)*6) - \
                           (((((season * 16) + match_round - 1) * 7) + 1) * 86400)

ht_date = HTdate()

def local2HT_date(t = None):
    return ht_date.from_local(t)

def HT2unix_date(_htdate):
    return ht_date.to_unix_time(_htdate)

class DataStore:
    """Conevnience class for handling data retrieved by XMLParse.get()."""

    comparison_key = None

    def __init__(self, data, single = False):
        """Takes data and initializes the class.

         - data -- the data dictionary.
         - single -- true if the data does not containn both display
                     and convert values."""

        self.data = data
        self.__single = single

    def get(self, key, def_val = None, display = False):
        """Return the value for the specified key.

        For single, the display argument is ignored.

         - key -- the specifed key.
         - def_val -- what to return if the key wasn't found.
         - display -- if true, return the display converted value."""

        d = self.data.get(key)

        if d is None:
            return def_val

        if self.__single:
            return d

        if display:
            return d[1]
        return d[0]

    def set_comparison_key(key):
        """Set the key to use as comparison key."""
        DataStore.comparison_key = key

    set_comparison_key = staticmethod(set_comparison_key)

    def __ne__(self, y):
        """Not equal with. x.__ne__(y) <=> x != y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) != y.get(self.comparison_key, None)

    def __eq__(self, y):
        """Equal with. x.__eq__(y) <=> x == y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) == y.get(self.comparison_key, None)

    def __le__(self, y):
        """Less than or equal. x.__le__(y) <=> x <= y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) <= y.get(self.comparison_key, None)

    def __lt__(self, y):
        """Less than. x.__le__(y) <=> x < y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) < y.get(self.comparison_key, None)

    def __ge__(self, y):
        """Greater than or equal. x.__le__(y) <=> x >= y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) >= y.get(self.comparison_key, None)

    def __gt__(self, y):
        """Greater than. x.__le__(y) <=> x > y"""

        if self.comparison_key is None:
            return False

        return self.get(self.comparison_key, None) > y.get(self.comparison_key, None)

    def __len__(self):
        """The amount of data in this storage."""

        return len(self.data)

    def __str__(self):
        """Make a string of this storage."""

        return str(self.data)

    def __repr__(self):
        """Make a string of this storage."""

        return repr(self.data)
