# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions and classes to help pulling out relevant information of
league details and league fixtures."""

from Common import *
import XMLParse

def team_cmp(t1, t2):
    """Compare which of two teams that are ahead of the other."""
    v = t2[6] - t1[6]
    if v == 0:
        v = (t2[4]-t2[5]) - (t1[4]-t1[5])
        if v == 0:
            v = t2[4] - t1[4]
            if v == 0:
                v = t1[8] - t2[8]
    return v

class Fixtures:
    """Class that extracts and stores relevant information about fixtures and
    leagues."""

    def __init__(self, llu_id, season, force_update = False):
        """Initialization.

         - llu_id -- The league level unit id of the league to get.
         - season -- The season to pull information for.
         - force_update -- Force the updating of the data from the server."""

        self.__extract_data(llu_id, season, force_update)

    def __extract_data(self, llu_id, season, force_update):
        """Extracts data from the database into better structures.

         - llu_id -- The league level unit id of the league to get.
         - season -- The season to pull information for.
         - force_update -- Force the updating of the data from the server."""

        data = XMLParse.get(["league fixtures", llu_id, season], force_update = force_update)

        self.league_id = data.get("LeagueLevelUnitID")
        self.league_name = data.get("LeagueLevelUnitName", "")
        self.season = data.get("Season", -1)

        match_indices = data.get("Match.xml_list", "").split(",")

        self.matches = {}
        for match in match_indices:
            id = data.get("Match+%s.MatchID" % match)
            round = data.get("Match+%s.MatchRound" % match)
            date = data.get("Match+%s.MatchDate" % match)

            hometeamid = data.get("Match+%s.HomeTeam.HomeTeamID" % match)
            hometeamname = data.get("Match+%s.HomeTeam.HomeTeamName" % match, "")
            homegoals = data.get("Match+%s.HomeGoals" % match)

            awayteamid = data.get("Match+%s.AwayTeam.AwayTeamID" % match)
            awayteamname = data.get("Match+%s.AwayTeam.AwayTeamName" % match, "")
            awaygoals = data.get("Match+%s.AwayGoals" % match)

            if round is None or id is None or hometeamid is None or awayteamid is None:
                break

            if homegoals is None or awaygoals is None:
                played = False
            else:
                played = True

            if not self.matches.has_key(round):
                self.matches[round] = {}

            self.matches[round][id] = (date,
                                       (hometeamid, hometeamname),
                                       (awayteamid, awayteamname),
                                       played,
                                       (homegoals, awaygoals))

        self.data = data

        # Get some information from the "table"-xml
        if self.league_id is None:
            self.league_level = None
            self.max_level = None
            self.country_id = None
            self.country_name = None
        else:
            data = XMLParse.get(["league details", self.league_id], force_update = force_update)

            self.league_level = data.get("LeagueLevel")
            self.max_level = data.get("MaxLevel")
            self.country_id = data.get("LeagueID")
            self.country_name = data.get("LeagueName", "")

        # Extract latest round
        self.latest_round = None
        for r in range(1,15):
            rmatches = self.matches.get(r)

            if rmatches != None:
                for m_id, vals in rmatches.items():
                    if not vals[3] and self.latest_round == None:
                        self.latest_round = r-1
                        break

        if self.latest_round == None:
            self.latest_round = 14


    def calc_table(self, round = 1000):
        """Calculates a table from the extracted data.

         - round -- Calculate a table for the given round.

         Returns three structures: (teams, hteams, ateams)

          - teams -- All matches for the teams.
          - hteams -- Only the home matches are counted for each team.
          - ateams -- Only the away matches are counted for each team.

         The structures themselves are a list of teams with a tuple of values
         about that team that looks like this: (pl, w, d, l, f, a, po, n, tid)

          - pl -- Number of played matches.
          - w -- Number of won matches.
          - d -- Number of drawn matches.
          - l -- Number of lost matches.
          - f -- Goals scored for the team.
          - a -- Goals scored against the team.
          - po -- The number of points for the team.
          - n -- The name of the team.
          - tid -- The id for the team."""

        eligible_rounds = filter(lambda x, r=round: 0 < x <= r, self.matches.keys())
        eligible_rounds.sort()

        def team_add(orig, add):
            res = []
            for i in range(7):
                res.append(orig[i] + add[i])
            res.append(add[7])
            return res

        # id: played, wins, draws, lost, goals_for, goals_against, points, name
        teams = {}
        hteams = {}
        ateams = {}

        for r in eligible_rounds:
            for m_id, vals in self.matches[r].items():
                (date, (hid, hname), (aid, aname), played, (hgoals, agoals)) = vals

                if not played:
                    hadd = [0, 0, 0, 0, 0, 0, 0, hname]
                    aadd = [0, 0, 0, 0, 0, 0, 0, aname]
                else:
                    if hgoals == agoals:
                        h_wdl = [0, 1, 0]
                        a_wdl = [0, 1, 0]
                        hp = 1
                        ap = 1
                    elif hgoals > agoals:
                        h_wdl = [1, 0, 0]
                        a_wdl = [0, 0, 1]
                        hp = 3
                        ap = 0
                    else:
                        h_wdl = [0, 0, 1]
                        a_wdl = [1, 0, 0]
                        hp = 0
                        ap = 3

                    hadd = [1] + h_wdl + [hgoals, agoals, hp, hname]
                    aadd = [1] + a_wdl + [agoals, hgoals, ap, aname]

                if not teams.has_key(hid):
                    teams[hid] = hadd
                else:
                    teams[hid] = team_add(teams[hid], hadd)

                # Home games
                if not hteams.has_key(hid):
                    hteams[hid] = hadd
                else:
                    hteams[hid] = team_add(hteams[hid], hadd)

                if not teams.has_key(aid):
                    teams[aid] = aadd
                else:
                    teams[aid] = team_add(teams[aid], aadd)

                # Away games
                if not ateams.has_key(aid):
                    ateams[aid] = aadd
                else:
                    ateams[aid] = team_add(ateams[aid], aadd)

        teams = [v + [k] for k, v in teams.items()]
        teams.sort(team_cmp)

        hteams = [v + [k] for k, v in hteams.items()]
        hteams.sort(team_cmp)

        ateams = [v + [k] for k, v in ateams.items()]
        ateams.sort(team_cmp)

        return (teams, hteams, ateams)

    def get_goal_hist(self):
        histf={}
        hista={}
        for r in self.matches.keys():
            for m_id, vals in self.matches[r].items():
                (date, (hid, hname), (aid, aname), played, (hgoals, agoals)) = vals
                if played:
                    if not histf.has_key(hid):
                        histf[hid]={}
                        hista[hid]={}
                    if not histf[hid].has_key(hgoals):
                        histf[hid][hgoals] = 1
                    else:
                        histf[hid][hgoals] += 1

                    if not hista[hid].has_key(agoals):
                        hista[hid][agoals] = 1
                    else:
                        hista[hid][agoals] += 1

                    if not histf.has_key(aid):
                        histf[aid]={}
                        hista[aid]={}
                    if not histf[aid].has_key(agoals):
                        histf[aid][agoals] = 1
                    else:
                        histf[aid][agoals] += 1

                    if not hista[aid].has_key(hgoals):
                        hista[aid][hgoals] = 1
                    else:
                        hista[aid][hgoals] += 1
        
        fdata={}
        adata={}
        for tid in histf.keys():
            fdata[tid]=[]
            adata[tid]=[]
            for goals in range(0, max(histf[tid].keys())+1):
                times = histf[tid].get(goals, 0)
                fdata[tid].append((str(goals), times))

            for goals in range(0, max(hista[tid].keys())+1):
                times = hista[tid].get(goals, 0)
                adata[tid].append((str(goals), times))

        return (fdata, adata)
    
    def write_table_text(self, outfile, round = 14):
        """Calculates and writes out a table on specified file.

         - outfile -- A file object to write on.
         - round -- The round to print the table for."""

        (teams, hteams, ateams) = self.calc_table(round)

        if self.league_id is None:
            outfile.write(_("No table to print") + "\n")
            return

        outfile.write(_("%s, season %d:") % (self.league_name, self.season) + "\n\n")

        outfile.write("%-30s %7s %2s %2s %2s %2s %3s - %3s %2s\n" % \
                      (_("Name"), _("Id"), _("Pl"), _("W"), _("D"),
                       _("L"), _("F"), _("A"), _("Po")))
        outfile.write("-"*63 + "\n")
        for pl, w, d, l, f, a, po, n, tid in teams:
            outfile.write("%-30s %7d %2d %2d %2d %2d %3d - %3d %2d\n" % \
                          (n[:30], tid, pl, w, d, l, f, a, po))
        outfile.flush()

    def GetLatestRound(self):
        return self.latest_round
