# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions and classes to be used by a plotting widget."""

from Common import *

from wxPython.wx import *

import math
import sys

PLOT_GRID_NONE = 0
PLOT_GRID_LINES = 1
PLOT_GRID_CHECKERED = 2

def merge_minmax(mm1, mm2):
    minmax = ()
    for i in [0,2]:
        minv1 = mm1[0+i]
        minv2 = mm2[0+i]
        if None in (minv1, minv2):
            if minv1 is not None:
                minmax += (minv1,)
            elif minv2 is not None:
                minmax += (minv2,)
            else:
                minmax += (None,)
        else:
            minmax += (min(minv1, minv2),)

        maxv1 = mm1[1+i]
        maxv2 = mm2[1+i]
        if None in (maxv1, maxv2):
            if maxv1 is not None:
                minmax += (maxv1,)
            elif maxv2 is not None:
                minmax += (maxv2,)
            else:
                minmax += (None,)
        else:
            minmax += (max(maxv1, maxv2),)

    return minmax


class PlotDataLine:
    """Store line data."""

    def __init__(self, data, line = True, marker = False, colour = None, data_range = (None, None)):
        self.__data = data
        self.__line = line
        self.__marker = marker
        self.__colour = colour
        self.__data_range = data_range
        self.__minmax = None

    def data(self):
        for d in self.__data:
            yield d

    def data_type(self):
        return "line"

    def use_line(self):
        return self.__line

    def use_marker(self):
        return self.__marker

    def get_colour(self):
        return self.__colour

    def get_data_range(self):
        return self.__data_range

    def get_minmax(self):
        if self.__minmax is None:
            if self.__data_range[0] is None:
                xs = zip(*self.__data)[0]
            else:
                xs = self.__data_range[0]

            if self.__data_range[1] is None:
                ys = zip(*self.__data)[1]
            else:
                ys = self.__data_range[1]

            self.__minmax = (min(xs), max(xs), min(ys), max(ys))
        return self.__minmax

class PlotDataHistogram:
    """Store histogram data."""

    def __init__(self, data, colour = None, data_range = None):
        self.__data = data
        self.__colour = colour
        self.__data_range = data_range
        self.__minmax = None

    def data(self):
        for d in self.__data:
            yield d

    def data_type(self):
        return "histogram"

    def get_colour(self):
        return self.__colour

    def get_data_range(self):
        return self.__data_range

    def get_xs(self):
        return zip(*self.__data)[0]

    def get_minmax(self):
        if self.__minmax is None:
            if self.__data_range is None:
                ys = zip(*self.__data)[1]
            else:
                ys = self.__data_range

            self.__minmax = (0.0, float(len(self.__data)), min(ys), max(ys))
        return self.__minmax


class PlotCtrl(wxWindow):
    """The control to be used by applications for plotting."""

    def __init__(self, parent, ID = -1,
                 pos = wxDefaultPosition, size = wxDefaultSize,
                 style = 0, name = 'PlotCtrl'):
        """Initialization of the plot control."""

        wxWindow.__init__(self, parent, ID, pos, size, style, name)

        EVT_PAINT(self, self.OnPaint)
        EVT_SIZE(self, self.OnSize)

        self.__border = (2, 2)
        self.__axes = (True, True, False, False)
        self.__grid_type = PLOT_GRID_LINES

        self.__plot_area_colour = (210, 210, 210)
        self.__plot_area_second_colour = (200, 200, 200)
        self.__grid_colour = (175, 175, 175)
        self.__data_colours = [(30, 30, 255),
                               (255, 30, 30),
                               (30, 255, 30)]
        self.__buffer = None
        self.__size = (None, None)

        self.__font_axes_size = 10
        self.__font_cache = {}

        self.__plot_data = []
        self.__x_ticks = None
        self.__y_ticks = None
        self.__x_force_all_ticks = False
        self.__y_force_all_ticks = False

        self.__reversed_x = False
        self.__reversed_y = False

    def AddData(self, dataname, data):
        self.__plot_data.append((dataname, data))

    def RemoveData(self, dataname):
        nlist = []
        for name, data in self.__plot_data:
            if name != dataname:
                nlist.append((name, data))
        self.__plot_data = nlist

    def ClearData(self):
        self.__plot_data = []

    def GetData(self, dataname):
        for name, data in self.__plot_data:
            if name == dataname:
                return data
        return None

    def SetTicks(self, ticks, vertical_axis, force_all_ticks = False):
        if vertical_axis:
            self.__y_ticks = ticks
            self.__y_force_all_ticks = force_all_ticks
        else:
            self.__x_ticks = ticks
            self.__x_force_all_ticks = force_all_ticks

    def GetTicks(self, vertical_axis):
        if vertical_axis:
            return self.__y_ticks
        return self.__x_ticks

    def RemoveTicks(self):
        self.__x_ticks = None
        self.__y_ticks = None
        self.__x_force_all_ticks = False
        self.__y_force_all_ticks = False

    def ReverseAxis(self, is_x, reverse = True):
        if is_x:
            self.__reversed_x = reverse
        else:
            self.__reversed_y = reverse

    def SetPlotAreaColour(self, col, col2 = None):
        self.__plot_area_colour = col
        if col2 is not None:
            self.__plot_area_second_colour = col2

    def SetGridColour(self, col):
        self.__grid_colour = col

    def SetGridType(self, _type):
        self.__grid_type = _type

    def Clear(self):
        dc = wxBufferedDC(wxClientDC(self), self.__buffer)

        bg_col = self.GetBackgroundColour()
        dc.SetBackground(wxBrush(bg_col))

        dc.Clear()

    def OnSize(self, event):
        self.__size = self.GetClientSizeTuple()
        self.__buffer = wxEmptyBitmap(*self.__size)

        self.__draw()

    def OnPaint(self, event):
        dc = wxBufferedPaintDC(self, self.__buffer)

    def Redraw(self):
        self.__draw()

    def __draw(self):
        dc = wxBufferedDC(wxClientDC(self), self.__buffer)

        bg_col = self.GetBackgroundColour()
        dc.SetBackground(wxBrush(bg_col))

        dc.Clear()

        data_type = None
        minmax = (None, None, None, None)
        for name, pd in self.__plot_data:
            minmax = merge_minmax(minmax, pd.get_minmax())
            if data_type is None:
                data_type = pd.data_type()
            elif data_type != pd.data_type():
                sys.stdout.write(_("Incompatible data types in plot, ignoring data '%s'.") % name)

        if data_type is None:
            data_type = "line"

        (xlower, xupper, ylower, yupper) = minmax
        if xlower is None:
            xlower = 0.0
        if xupper is None:
            xupper = 1.0
        if ylower is None:
            ylower = 0.0
        if yupper is None:
            yupper = 1.0
        if xlower == xupper:
            xlower -= 1.0
            xupper += 1.0
        if ylower == yupper:
            ylower -= 1.0
            yupper += 1.0

        minmax = (xlower, xupper, ylower, yupper)

        if data_type == "histogram":
            all_xs = []
            for name, pd in self.__plot_data:
                if pd.data_type() == "histogram":
                    for x in pd.get_xs():
                        if x not in all_xs:
                            all_xs.append(x)

        orig_border = self.__border + self.__border
        new_border = self.__border + self.__border
        current_border = (None, None, None, None)

        while current_border != new_border:
            current_border = new_border

            pa_area = current_border[:2] + \
                      (self.__size[0] - (current_border[0] + current_border[2]),
                       self.__size[1] - (current_border[1] + current_border[3]))

            if data_type == "line":
                if self.__x_force_all_ticks and self.__x_ticks is not None:
                    xticks = self.__x_ticks
                    xlower = min(zip(*xticks)[0])
                    xupper = max(zip(*xticks)[0])
                else:
                    nx_ticks = self.__calc_n_ticks(pa_area[2])
                    xlower, xupper, xticks = self.__get_ticks(minmax[0], minmax[1], nx_ticks, True)
            elif data_type == "histogram":
                lenx = len(all_xs)
                xlower = 0.0
                xupper = float(lenx)
                xticks = [(float(x) + 0.5, all_xs[x]) for x in range(lenx)]
                xticks_grid = [float(x) for x in range(lenx+1)]

            if self.__y_force_all_ticks and self.__y_ticks is not None:
                yticks = self.__y_ticks
                ylower = min(zip(*yticks)[0])
                yupper = max(zip(*yticks)[0])
            else:
                ny_ticks = self.__calc_n_ticks(pa_area[3])
                ylower, yupper, yticks = self.__get_ticks(minmax[2], minmax[3], ny_ticks, False)

            minmax = (xlower, xupper, ylower, yupper)

            xd_sizes = [(xd, dc.GetTextExtent(xd)) for xt, xd in xticks]
            yd_sizes = [(yd, dc.GetTextExtent(yd)) for yt, yd in yticks]

            xtmp = zip(*xd_sizes)[1]
            ytmp = zip(*yd_sizes)[1]

            xd_max_width = max(zip(*xtmp)[0]) + 4
            xd_max_height = max(zip(*xtmp)[1]) + 4
            yd_max_width = max(zip(*ytmp)[0]) + 4
            yd_max_height = max(zip(*ytmp)[1]) + 4

            xticks_pt = [self.__value_to_point(xt, minmax[0], minmax[1] - minmax[0],
                                               pa_area[2], pa_area[0], False)
                         for xt, xd in xticks]

            n_descr_levels = 1

            x_tmp1 = xticks_pt[0]
            xdiff = None
            for x_tmp2 in xticks_pt[1:]:
                if xdiff is None:
                    xdiff = abs(x_tmp2 - x_tmp1)
                else:
                    xdiff = min(abs(x_tmp2 - x_tmp1), xdiff)
                x_tmp1 = x_tmp2

            if xdiff < xd_max_width:
                n_descr_levels = 2

            new_border = (orig_border[0] + yd_max_width,
                          orig_border[1] + xd_max_height // 2 + 1,
                          orig_border[2] + yd_max_width // 2 + 1,
                          orig_border[3] + xd_max_height * n_descr_levels)

        yticks_pt = [self.__value_to_point(yt, minmax[2], minmax[3] - minmax[2],
                                           pa_area[3], pa_area[1], True)
                     for yt, yd in yticks]

        if data_type == "histogram":
            xticks_grid_pt = [self.__value_to_point(xt, minmax[0], minmax[1] - minmax[0],
                                                    pa_area[2], pa_area[0], False)
                              for xt in xticks_grid]
            self.__draw_area(dc, minmax, pa_area, xticks_grid_pt, yticks_pt, True, True)
        else:
            self.__draw_area(dc, minmax, pa_area, xticks_pt, yticks_pt, True, True)

        self.__draw_axes(dc, minmax, pa_area, xticks_pt, yticks_pt)

        self.__draw_axes_descriptions(dc, minmax, pa_area, xticks_pt, yticks_pt,
                                      xd_sizes, yd_sizes, n_descr_levels)

        if data_type == "line":
            self.__draw_lines(dc, minmax, pa_area)
        elif data_type == "histogram":
            self.__draw_histograms(dc, minmax, pa_area, all_xs, xticks_pt)

    def __draw_area(self, dc, minmax, pa_area, xticks_pt, yticks_pt,
                    horizontal, vertical):
        if self.__grid_type == PLOT_GRID_NONE:
            self.__draw_area_none(dc, minmax, pa_area, xticks_pt, yticks_pt,
                                  horizontal, vertical)

        elif self.__grid_type == PLOT_GRID_LINES:
            self.__draw_area_lines(dc, minmax, pa_area, xticks_pt, yticks_pt,
                                   horizontal, vertical)

        elif self.__grid_type == PLOT_GRID_CHECKERED:
            self.__draw_area_checkered(dc, minmax, pa_area, xticks_pt, yticks_pt,
                                       horizontal, vertical)

    def __draw_area_none(self, dc, minmax, pa_area, xticks_pt, yticks_pt,
                         horizontal, vertical):
        dc.SetPen(wxPen(self.__plot_area_colour))
        dc.SetBrush(wxBrush(self.__plot_area_colour))
        dc.DrawRectangle(*pa_area)

    def __draw_area_lines(self, dc, minmax, pa_area, xticks_pt, yticks_pt,
                          horizontal, vertical):
        dc.SetPen(wxPen(self.__plot_area_colour))
        dc.SetBrush(wxBrush(self.__plot_area_colour))
        dc.DrawRectangle(*pa_area)

        dc.SetPen(wxPen(self.__grid_colour))
        if vertical:
            for x in xticks_pt:
                dc.DrawLine(x, pa_area[1],
                            x, pa_area[1] + pa_area[3])

        if horizontal:
            for y in yticks_pt:
                dc.DrawLine(pa_area[0], y,
                            pa_area[0] + pa_area[2], y)

    def __draw_area_checkered(self, dc, minmax, pa_area, xticks_pt, yticks_pt,
                              horizontal, vertical):
        pens = [wxPen(self.__plot_area_colour),
                wxPen(self.__plot_area_second_colour)]
        brushes = [wxBrush(self.__plot_area_colour),
                   wxBrush(self.__plot_area_second_colour)]

        if vertical:
            nxt = xticks_pt + []
            if self.__reversed_x:
                if xticks_pt[0] != pa_area[0] + pa_area[2]:
                    nxt = [pa_area[0] + pa_area[2]] + nxt
                if xticks_pt[-1] != pa_area[0]:
                    nxt = nxt + [pa_area[0]]
            else:
                if xticks_pt[0] != pa_area[0]:
                    nxt = [pa_area[0]] + nxt
                if xticks_pt[-1] != pa_area[0] + pa_area[2]:
                    nxt = nxt + [pa_area[0] + pa_area[2]]
        else:
            nxt = [pa_area[0], pa_area[0] + pa_area[2]]

        if horizontal:
            nyt = yticks_pt + []
            if self.__reversed_y:
                if yticks_pt[-1] != pa_area[1] + pa_area[3]:
                    nyt = nyt + [pa_area[1] + pa_area[3]]
                if yticks_pt[0] != pa_area[1]:
                    nyt = [pa_area[1]] + nyt
            else:
                if yticks_pt[-1] != pa_area[1]:
                    nyt = nyt + [pa_area[1]]
                if yticks_pt[0] != pa_area[1] + pa_area[3]:
                    nyt = [pa_area[1] + pa_area[3]] + nyt
        else:
            nyt = [pa_area[1], pa_area[1] + pa_area[3]]

        lx = nxt[0]
        ly = nyt[0]
        i = 0
        for rx in nxt[1:]:
            col = i % 2
            for ry in nyt[1:]:
                col = (col + 1) % 2

                dc.SetPen(pens[col])
                dc.SetBrush(brushes[col])
                dc.DrawRectangle(lx, ry, rx - lx + 1, ly - ry + 1)

                ly = ry
            lx = rx
            ly = nyt[0]
            i += 1

    def __draw_axes(self, dc, minmax, pa_area, xticks_pt, yticks_pt):
        dot_size = 2

        corners = [(pa_area[0] + pa_area[2], pa_area[1] + pa_area[3]),
                   (pa_area[0], pa_area[1] + pa_area[3]),
                   (pa_area[0], pa_area[1]),
                   (pa_area[0] + pa_area[2], pa_area[1])]

        i = 0
        dc.SetPen(wxPen((0,0,0)))
        for axis in self.__axes:
            if axis:
                dc.DrawLine(*(corners[i] + corners[(i + 1) % 4]))
                if bool(i % 2):
                    s = pa_area[3]
                    xpoint = (lambda e, d, p: corners[i][0] + d * dot_size + p)
                    ypoint = (lambda e, d, p: e)
                    nt = yticks_pt
                else:
                    s = pa_area[2]
                    xpoint = (lambda e, d, p: e)
                    ypoint = (lambda e, d, p: corners[i][1] + d * dot_size + p)
                    nt = xticks_pt

                for e in nt:
                    dc.DrawLine(xpoint(e, -1, 0), ypoint(e, -1, 0),
                                xpoint(e, 1, 1), ypoint(e, 1, 1))

            i += 1

    def __draw_axes_descriptions(self, dc, minmax, pa_area, xticks_pt, yticks_pt,
                                 xd_sizes, yd_sizes, n_levels):
        dc.SetFont(self.__get_font(self.__font_axes_size))

        dc.SetTextForeground((0,0,0))
        i = 0
        level = 0
        for xd, (xd_ex, xd_ey) in xd_sizes:
            x = xticks_pt[i]
            y = pa_area[1] + pa_area[3]
            dc.DrawText(xd, x - xd_ex/2, y + ((xd_ey + 4) * level) + 4)
            i += 1
            level = (level + 1) % n_levels

        i = 0
        for yd, (yd_ex, yd_ey) in yd_sizes:
            x = pa_area[0]
            y = yticks_pt[i]
            dc.DrawText(yd, x - (yd_ex + 4), y - yd_ey/2)
            i += 1

    def __draw_lines(self, dc, minmax, pa_area):
        dwidth = (minmax[1] - minmax[0])
        dheight = (minmax[3] - minmax[2])

        i = 0
        dc.SetClippingRegion(pa_area[0], pa_area[1], pa_area[2] + 1, pa_area[3] + 1)
        for name, pd in self.__plot_data:
            if pd.get_colour() is None:
                colour = self.__data_colours[i]
                i = (i + 1) % len(self.__data_colours)
            else:
                colour = pd.get_colour()

            dc.SetPen(wxPen(colour, 3))
            lx = None
            ly = None
            for x, y in pd.data():
                rx = self.__value_to_point(x, minmax[0], dwidth, pa_area[2], pa_area[0], False)
                ry = self.__value_to_point(y, minmax[2], dheight, pa_area[3], pa_area[1], True)

                if pd.use_line() and lx is not None:
                    dc.DrawLine(lx, ly, rx, ry)

                if pd.use_marker():
                    dc.DrawLine(rx - 3, ry - 3, rx + 4, ry + 4)
                    dc.DrawLine(rx - 3, ry + 3, rx + 4, ry - 4)

                lx = rx
                ly = ry
        dc.DestroyClippingRegion()

    def __draw_histograms(self, dc, minmax, pa_area, all_xs, xticks_pt):
        dwidth = (minmax[1] - minmax[0])
        dheight = (minmax[3] - minmax[2])

        pd_dicts = []
        for name, pd in self.__plot_data:
            if pd.data_type() == "histogram":
                pd_dicts.append((pd, dict(pd.data())))

        space = 4

        xs = []
        for xi in range(len(all_xs)):
            xd = all_xs[xi]
            if xi == 0:
                x1 = pa_area[0] + space
            else:
                x1 = int((xticks_pt[xi] + xticks_pt[xi-1]) / 2.0) + space

            if xi == len(all_xs) - 1:
                x2 = pa_area[0] + pa_area[2] - space
            else:
                x2 = int((xticks_pt[xi+1] + xticks_pt[xi]) / 2.0) - space

            xs.append((xd, x1, x2))

        dc.SetClippingRegion(pa_area[0], pa_area[1], pa_area[2] + 1, pa_area[3] + 1)
        for xd, x1, x2 in xs:
            col_no = 0
            i = 0
            for pd, pdd in pd_dicts:
                if pd.get_colour() is None:
                    colour = self.__data_colours[col_no]
                    col_no = (col_no + 1) % len(self.__data_colours)
                else:
                    colour = pd.get_colour()

                dc.SetPen(wxPen((0,0,0)))
                dc.SetBrush(wxBrush(colour))

                yval = pdd.get(xd, 0)
                y0 = self.__value_to_point(0.0, minmax[2], dheight, pa_area[3], pa_area[1], True)
                yv = self.__value_to_point(float(yval), minmax[2], dheight,
                                           pa_area[3], pa_area[1], True)
                rx1 = x1 + int(float(i) * float(x2 - x1) / float(len(pd_dicts)))
                rx2 = x1 + int(float(i+1) * float(x2 - x1) / float(len(pd_dicts)))
                dc.DrawRectangle(rx1, y0, rx2 - rx1 + 1, yv - y0 + 1)
                i += 1


        dc.DestroyClippingRegion()

    def __value_to_point(self, val, minval, width, pa_size, pa_min, is_y):
        if (is_y and not self.__reversed_y) or (not is_y and self.__reversed_x):
            return pa_min + pa_size - int(round(((val - minval) / width) * pa_size))
        return pa_min + int(round(((val - minval) / width) * pa_size))

    def __calc_n_ticks(self, size):
        ds_ideal = 80.0
        n_dot_list = [1, 2, 4, 6, 8, 10, 12, 15, 20, 25, 30, 40, 50]

        dsizes = [(float(size)/float(d), d) for d in n_dot_list]
        dsizes = [(abs(val - ds_ideal), d) for val, d in dsizes]
        return min(dsizes)[1]

    def __get_ticks(self, lower, upper, n_ticks, is_x):
        if is_x:
            ts = self.__x_ticks
        else:
            ts = self.__y_ticks

        if ts is not None:
            lower = min(zip(*ts)[0] + (lower,))
            upper = max(zip(*ts)[0] + (upper,))
            c_ticks = self.__calc_ticks(0, len(ts) - 1, n_ticks, True)
            ticks = []
            for i in c_ticks:
                ticks.append(ts[i])
        else:
            ticks = self.__calc_ticks(lower, upper, n_ticks, False)
        return (lower, upper, ticks)

    def __calc_ticks(self, lower, upper, n_ticks, use_int = False):
        if getattr(self, "__multiples", None) is None:
            mult = [2.0, 3.0, 4.0, 5.0, 6.0, 8.0]
            self.__multiples = [(m, math.log10(m)) for m in mult]

        ideal = (upper - lower)/float(n_ticks)

        log = math.log10(ideal)
        power = math.floor(log)
        fraction = log - power

        factor = 1.0
        error = fraction

        for f, logf in self.__multiples:
            e = abs(fraction - logf)
            if e < error:
                error = e
                factor = f

        if not use_int:
            if abs(power) > 6:
                format = '%+7.1e'
            elif power >= 0:
                digits = max(1, int(power))
                format = "%%%d.0f" % digits
            else:
                digits = abs(int(power))
                format = "%%%d.%df" % (digits + 2, digits)

        grid = factor * 10. ** power
        if use_int:
            grid = round(grid)
            if grid <= 0.0:
                grid = 1.0
        tick = -grid * math.floor(-lower/grid)

        ticks = []
        while tick <= upper:
            if tick == -0.0:
                tick = 0.0
            if use_int:
                ticks.append(int(tick))
            else:
                ticks.append((tick, format % tick))
            tick = tick + grid
        return ticks

    def __get_font(self, size):
        of = self.GetFont()

        key = (size, of.GetFamily(), of.GetStyle(), of.GetWeight())
        font = self.__font_cache.get(key, None)

        if font is None:
            font = wxFont(*key)
            self.__font_cache[key] = font
        return font

def __test():

    class AppFrame(wxFrame):
        def __init__(self, parent, id, title):
            wxFrame.__init__(self, parent, id, title)

            sizer = wxBoxSizer(wxVERTICAL)

            self.pc = PlotCtrl(self)
            #self.pc.SetBackgroundColour("#FFFFFF")
            #self.pc.SetPlotAreaColour("#D0D0D0")
            self.pc.SetGridType(PLOT_GRID_CHECKERED)

            n_r = 400
            max_r = 2*math.pi
            plot_data_t = [float(x)/n_r*max_r for x in range(n_r + 1)]
            plot_data_1 = [(x, math.sin(x)) for x in plot_data_t]
            plot_data_2 = [(x, math.cos(x)) for x in plot_data_t]
            plot_data_3 = [(x, math.sin(x*8.0)*(x/4.0)) for x in plot_data_t]
            plot_data_4 = [(x, x/4.0) for x in plot_data_t]
            plot_data_5 = [(x, -x/4.0) for x in plot_data_t]

            plot_data_1x = [(x, math.sin(x)) for x in plot_data_t[::40]]

            self.pc.AddData("sinus", PlotDataLine(plot_data_1, colour = (255, 255, 30)))
            self.pc.AddData("sline1", PlotDataLine(plot_data_4, colour = (30, 30, 30)))
            self.pc.AddData("sline2", PlotDataLine(plot_data_5, colour = (30, 30, 30)))
            self.pc.AddData("cosinus", PlotDataLine(plot_data_2))
            self.pc.AddData("sin*x", PlotDataLine(plot_data_3))
            self.pc.AddData("sin_markers", PlotDataLine(plot_data_1x, line = False,
                                                          marker = True, colour = (30, 30, 255)))

            self.pc.SetTicks([(0, "0"), (math.pi/4.0, "pi/4"),
                              (math.pi/2.0, "pi/2"), (3.0*math.pi/4.0, "3/4*pi"),
                              (math.pi, "pi"),  (5.0*math.pi/4.0, "5/4*pi"),
                              (3.0/2.0*math.pi, "3/2*pi"),  (7.0*math.pi/4.0, "7/4*pi"),
                              (2.0*math.pi, "2*pi")], False)

            sizer.Add(self.pc, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

            self.pch = PlotCtrl(self)
            hdata1 = [("disastrous", 11), ("wretched", 9), ("poor", 0),
                      ("weak", -2), ("inadequate", 5), ("passable", 6)]

            hdata2 = [("disastrous", 1), ("wretched", 7), ("poor", 3),
                      ("weak", 4), ("inadequate", 2), ("passable", 10)]

            hdata3 = [("wretched", 3), ("weak", 2), ("inadequate", 2),
                      ("passable", 8), ("solid", 5), ("exellent", 4)]

            self.pch.SetGridType(PLOT_GRID_CHECKERED)
            #self.pch.SetBackgroundColour("#FFFFFF")
            self.pch.AddData("skill3", PlotDataHistogram(hdata1))
            self.pch.AddData("skill2", PlotDataHistogram(hdata2))
            self.pch.AddData("skill1", PlotDataHistogram(hdata3, colour = (255, 255, 30)))

            sizer.Add(self.pch, 1, wxEXPAND | wxALIGN_CENTRE | wxALL, 5)

            self.SetSizer(sizer)
            self.SetAutoLayout(True)
            sizer.Fit(self)

            self.SetClientSize((400, 800))

    class MyApp(wxApp):
        def OnInit(self):
            frame = AppFrame(wx.NULL, -1, "PlotCtrl")
            frame.Centre()
            frame.Show(True)
            self.SetTopWindow(frame)
            return True

    app = MyApp(0)
    app.MainLoop()

if __name__ == '__main__':
    __test()
