# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Functions and classes for evaluating teams in different ways."""

from Common import *
import Terms
import Config

import operator

class PlayerEvaluation:
    """Class that defines a number of functions and variables to asses a
    players value."""

    eval_positions = {
        "goalie": [("KeeperSkill", 1.0)],

        "centraldefender":  [("DefenderSkill", 0.88),
                             ("StaminaSkill", 0.06),
                             ("PlaymakerSkill", 0.06)],

        "centraldefender_offensive": [("DefenderSkill", 0.72),
                                      ("StaminaSkill", 0.08),
                                      ("PlaymakerSkill", 0.20)],

        "centraldefender_towards_the_wing": [("DefenderSkill", 0.60),
                                             ("StaminaSkill", 0.06),
                                             ("PlaymakerSkill", 0.04),
                                             ("WingerSkill", 0.30)],

        "wingback": [("DefenderSkill", 0.52),
                     ("WingerSkill", 0.32),
                     ("StaminaSkill", 0.06),
                     ("PlaymakerSkill", 0.06),
                     ("PassingSkill", 0.06)],

        "wingback_offensive": [("DefenderSkill", 0.42),
                               ("WingerSkill", 0.40),
                               ("StaminaSkill", 0.06),
                               ("PlaymakerSkill", 0.12)],

        "wingback_defensive": [("DefenderSkill", 0.64),
                               ("WingerSkill", 0.24),
                               ("StaminaSkill", 0.06),
                               ("PlaymakerSkill", 0.06)],

        "wingback_towards_the_middle": [("DefenderSkill", 0.60),
                                        ("WingerSkill", 0.26),
                                        ("StaminaSkill", 0.06),
                                        ("PlaymakerSkill", 0.06),
                                        ("PassingSkill", 0.02)],

        "innermid": [("PlaymakerSkill", 0.54),
                     ("StaminaSkill", 0.20),
                     ("PassingSkill", 0.12),
                     ("DefenderSkill", 0.14)],

        "innermid_offensive": [("PlaymakerSkill", 0.46),
                               ("StaminaSkill", 0.20),
                               ("PassingSkill", 0.22),
                               ("DefenderSkill", 0.12)],

        "innermid_defensive": [("PlaymakerSkill", 0.46),
                               ("StaminaSkill", 0.20),
                               ("PassingSkill", 0.12),
                               ("DefenderSkill", 0.22)],

        "innermid_towards_the_wing": [("PlaymakerSkill", 0.42),
                                      ("StaminaSkill", 0.16),
                                      ("WingerSkill", 0.16),
                                      ("PassingSkill", 0.12),
                                      ("DefenderSkill", 0.14)],

        "winger": [("PlaymakerSkill", 0.34),
                   ("StaminaSkill", 0.12),
                   ("PassingSkill", 0.14),
                   ("DefenderSkill", 0.14),
                   ("WingerSkill", 0.26)],

        "winger_offensive": [("PlaymakerSkill", 0.20),
                             ("StaminaSkill", 0.10),
                             ("PassingSkill", 0.24),
                             ("DefenderSkill", 0.06),
                             ("WingerSkill", 0.40)],

        "winger_defensive": [("PlaymakerSkill", 0.26),
                             ("StaminaSkill", 0.14),
                             ("PassingSkill", 0.10),
                             ("DefenderSkill", 0.30),
                             ("WingerSkill", 0.20)],

        "winger_towards_the_middle": [("PlaymakerSkill", 0.44),
                                      ("StaminaSkill", 0.14),
                                      ("PassingSkill", 0.12),
                                      ("DefenderSkill", 0.12),
                                      ("WingerSkill", 0.18)],

        "forward": [("ScorerSkill", 0.64),
                    ("PassingSkill", 0.36)],

        "forward_defensive": [("ScorerSkill", 0.48),
                              ("PassingSkill", 0.28),
                              ("PlaymakerSkill", 0.16),
                              ("StaminaSkill", 0.08)],
        }

    injury_modification = [0.85, 0.1]
    card_modification = 0.1

    form_modification = [0.75, 16.0]

    def player_eval(position, player):
        """Calculate the player evaluation.

         - position -- The position of the player.
         - player -- The player data to evaluate."""

        attrs = PlayerEvaluation.eval_positions[position]
        basic = reduce(operator.add, [player.get(attr, 1.0)*weight for attr, weight in attrs])
        form = player.get("PlayerForm", 1.0)
        injury = player.get("InjuryLevel", -1)
        cards = player.get("Cards", 0)
        mod = 1.0
        if Config.config.get("Player evaluation", "ignore injury") == False:
            if injury == 0:
                mod = PlayerEvaluation.injury_modification[0]
            elif injury > 0:
                mod = PlayerEvaluation.injury_modification[1]
        if Config.config.get("Player evaluation", "ignore cards") == False:
            if cards == 3:
                mod = PlayerEvaluation.card_modification
        if Config.config.get("Player evaluation", "ignore form") == True:
            form = 6
        form_mod = (PlayerEvaluation.form_modification[0] +
                    form / PlayerEvaluation.form_modification[1])
        return basic * form_mod * mod

    player_eval = staticmethod(player_eval)

class TeamEvaluation:
    """Class that defines a number of functions and variables to asses the
    value of a team and to select lineups."""

    def calculate_player_table(all_players):
        """Calculate all player evaluations and sort them in a big table."""
        p_eval = PlayerEvaluation.player_eval

        player_table = {}
        for i, (position, descr) in Terms.team_positions.items():
            position_list = []
            for pid, player in all_players.items():
                position_list.append((p_eval(position, player), pid))

            position_list.sort()
            position_list.reverse()
            player_table[i] = position_list

        return player_table

    calculate_player_table = staticmethod(calculate_player_table)
