# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Lists of xml files and how to parse the information gathered inside
them.

The following module-global variables exist here:

 * xml_known_xmls

   A dictionary with HT filenames as keys. The values consists of a tuple
   with the following content: (mattr_name, common, memory_only,
                                identification_creation,
                                default_identificaiton,
                                id2string_conversion,
                                HT_arguments,
                                make_list,
                                xml_description)

    - mattr_name -- The Matrricks name of the xml.
    - common -- If the xml belongs to the common database or not.
    - memory_only -- If the xml should be stored in memory or on disk.
    - identification_creation -- A list of pairs for creating
                                 identification from the xml. The
                                 first value in the pair should be a
                                 path to the element in the xml and the
                                 second shoiuld be a conversion function.
    - default_identificaiton -- A function that returns a default
                                identification for the xml if no other is
                                given.
    - id2string_conversion -- A function that takes an identification as
                              argument and returns a string with that
                              information. Used as keys in the database
                              plus as filenames when storing xmls.
    - HT_arguments -- A function returning a list of pairs containing
                      arguments used when downloading the xml. Takes the
                      identification as argument.
    - make_list -- Boolean value to tell if there should be made a list
                   of all xml:s stored in the database. The key to this
                   list will be named mattr_name.xml_list.
    - xml_description -- A description of the xml to be displayed on
                         screen.
    - less_fn -- Function to compare two id:s with each other.
                 If this is none, operator.lt() is used.
                 Also note that the default comparison compares the
                 idstrings and not the tuple-form. If you anything
                 else is wanted, a less_fn has to be defined.

 * xml_name_table

   A mapping of the Mattricks names of the xml:s to HT filenames.

 * xml_known_data

   A dictionary containing all xml:s and their known data. Has the
   Mattricks name as keys and as values are dictionaries with the
   following content: element_path: (content_descr, content_conv, disp_conv)

    - element_path -- The path to the element in question. A path consists
                      of elements that in the xml was inside each other. I.e.
                      a path with 'Team.TeamID' Looked like this in the xml:

                      <Team>
                         <TeamID>1234567</TeamID>
                      </Team>

    - content_descr -- A description of the content of this path.
                       Can be None.
    - content_conv -- How to convert this to a more python-friendly form
                      than pure strings.
    - disp_conv -- A second conversion (performed after the content_conv
                   conversion) to be used when displaying the data.
"""

# Format of xml_known_data:
# {
# "data_section": {
#    "element.path": (content_description, content_conversion, content_display_conversion),
#    ...}
# ...}

from Common import *

from Database import databases
import Terms
import Countries
import CommonData

import string
import time

def live_identifier_to_str(identifier):
    """The id2string conversion for HT live."""
    li = len(identifier)
    if li == 3:
        return "%s-a%s-m%d" % (identifier[0], identifier[1], identifier[2])
    elif li == 2:
        return "%s-a%s" % (identifier[0], identifier[1])
    else:
        return "%s" % (identifier[0],)

def live_fetch_args(identifier):
    """Returns the arguments when fetching HT live."""
    li = len(identifier)
    if li == 3:
        return [("actionType", identifier[1]),
                ("matchID", str(identifier[2]))]
    elif li == 2:
        return [("actionType", identifier[1])]
    else:
        return []

def team_details_fetch_args(identifier):
    """Returns the arguments when fetching team details."""
    if identifier[1] < 0:
        return []
    return [("teamID", identifier[1])]

def player_fetch_args(identifier):
    """Returns the arguments when fetching players."""
    if identifier[1] == CommonData.get_current_teamid():
        return []
    return [("teamID", identifier[1])]

def leagueFixtures_fetch_args(identifier):
    """Returns the arguments when fetching league fixtures."""
    args = []
    if identifier[1] > 0:
        args.append(("leagueLevelUnitID", str(identifier[1])))
    if identifier[2] > 0:
        args.append(("season", str(identifier[2])))
    return args

def transfer_history_id_to_str(identifier):
    """The id2string conversion for transfer history."""
    if identifier[1] == "team":
        return "%s-%s-t%d" % (identifier[0], identifier[1], identifier[2])
    elif identifier[1] == "player":
        return "%s-%s-p%d" % (identifier[0], identifier[1], identifier[2])
    return "%s-%s" % (identifier[0], identifier[1])

def transfer_history_fetch_args(identifier):
    """Returns the arguments when fetching transfer history."""
    args = [("actionType", identifier[1])]
    if identifier[1] == "team":
        args.append(("teamID", str(identifier[2])))
        if len(identifier) > 3:
            args.append(("endDate", mkHTtimestring(identifier[3])))
    elif identifier[1] == "player":
        args.append(("playerID", str(identifier[2])))
    return args

# Format of xml_known_xmls:
# {
# "filename": ("data_section",
#              in_common_section, only_in_memory,
#              [(identification_element_path, identification_conversion), ...],
#              (default_identification,),
#              id2string-conversion,
#              get_fetch_args,
#              make_list,
#              xml_description)
#              )
# }
xml_known_xmls = {
    "unknown": ("unknown",
                True, True,
                [("FetchedDate", time2ymd)],
                (lambda : [time.localtime(time.time())[0:3]]),
                (lambda x: "%s-%04d-%02d-%02d" % ((x[0],) + x[1])),
                (lambda identifier: None),
                False,
                _("unknown data"),
                None,
                ),

    "teamDetails.asp": ("team details",
                        True, False,
                        [("Team.TeamID", None)],
                        (lambda : None),
                        (lambda x: "%s-t%d" % (x[0], x[1])),
                        team_details_fetch_args,
                        False,
                        _("team details"),
                        None,
                        ),

    "leagueDetails.asp": ("league details",
                          True, False,
                          [("LeagueLevelUnitID", None)],
                          (lambda : None),
                          (lambda x: "%s-l%d" % (x[0], x[1])),
                          (lambda x: [("leagueLevelUnitID", str(x[1]))]),
                          False,
                          _("league details"),
                          None,
                          ),

    "leagueFixtures.asp": ("league fixtures",
                           True, False,
                           [("LeagueLevelUnitID", None),
                            ("Season", None)],
                           (lambda : None),
                           (lambda x: "%s-l%d-s%d" % (x[0], x[1], x[2])),
                           leagueFixtures_fetch_args,
                           False,
                           _("league fixtures"),
                           None,
                           ),

    "matches.asp": ("match list",
                    True, False,
                    [("Team.TeamID", None)],
                    (lambda : None),
                    (lambda x: "%s-t%d" % (x[0], x[1])),
                    (lambda x: [("teamID", str(x[1]))]),
                    False,
                    _("current match list"),
                    None,
                    ),

    "matchLineup.asp": ("match lineup",
                        True, False,
                        [("MatchID", None),
                         ("Team.TeamID", None)],
                        (lambda : None),
                        (lambda x: "%s-m%d-t%d" % (x[0], x[1], x[2])),
                        (lambda x: [("matchID", str(x[1])),
                                    ("teamID", str(x[2]))]),
                        False,
                        _("match lineup"),
                        None,
                        ),

    "matchOrders.asp": ("match orders",
                        True, False,
                        [("MatchID", None)],
                        (lambda : None),
                        (lambda x: "%s-m%d" % (x[0], x[1])),
                        (lambda x: [("matchID", str(x[1]))]),
                        False,
                        _("match orders"),
                        None,
                        ),

    "matchDetails.asp": ("match details",
                         True, False,
                         [("Match.MatchID", None)],
                         (lambda : None),
                         (lambda x: "%s-m%d" % (x[0], x[1])),
                         (lambda x: [("matchID", str(x[1]))]),
                         False,
                         _("match details"),
                         None,
                         ),

    "live.asp": ("HT live",
                 True, True,
                 [],
                 (lambda : []),
                 live_identifier_to_str,
                 live_fetch_args,
                 False,
                 _("live match data"),
                 None,
                 ),

    "worldDetails.asp": ("world details",
                         True, False,
                         [],
                         (lambda : []),
                         (lambda x: "%s" % x[0]),
                         (lambda x: [("actionType", "leagues")]),
                         False,
                         _("world details"),
                         None,
                         ),

    "bookmarks.asp": ("bookmarks",
                      True, True,
                      [],
                      (lambda : []),
                      (lambda x: "%s" % x[0]),
                      (lambda x: [("actionType", "all")]),
                      False,
                      _("HT bookmarks"),
                      None,
                      ),

    "players.asp": ("all players",
                    False, False,
                    [("Team.TeamID", None),
                     ("FetchedDate", (lambda t: time.localtime(t)[0:4]))],
                    (lambda : [CommonData.get_current_teamid(),
                               time.localtime(time.time())[0:4]]),
                    (lambda x: "%s-t%d-%04d-%02d-%02d-%02d" % ((x[0], x[1]) + x[2])),
                    player_fetch_args,
                    True,
                    _("data for all players"),
                    None,
                    ),

    "transferHistory.asp": ("transfer history",
                            True, False,
                            [("ActionType", None),
                             ("Team.TeamID", None),
                             ("Transfers.Player.PlayerID", None)],
                            (lambda : ["team", CommonData.get_current_teamid()]),
                            transfer_history_id_to_str,
                            transfer_history_fetch_args,
                            False,
                            _("transfer history"),
                            None,
                            ),

    "economy.asp": ("economy",
                    False, False,
                    [("FetchedDate", (lambda t: time.localtime(t)[0:4]))],
                    (lambda : [time.localtime(time.time())[0:4]]),
                    (lambda x: "%s-%04d-%02d-%02d-%02d" % ((x[0],) + x[1])),
                    (lambda x: []),
                    True,
                    _("economy data"),
                    None,
                    ),

    "training.asp": ("training",
                     False, False,
                     [("FetchedDate", (lambda t: time.localtime(t)[0:4]))],
                     (lambda : [time.localtime(time.time())[0:4]]),
                     (lambda x: "%s-%04d-%02d-%02d-%02d" % ((x[0],) + x[1])),
                     (lambda x: []),
                     True,
                     _("training data"),
                     None,
                    ),

    "club.asp": ("club",
                 False, False,
                 [("FetchedDate", (lambda t: time.localtime(t)[0:4]))],
                 (lambda : [time.localtime(time.time())[0:4]]),
                 (lambda x: "%s-%04d-%02d-%02d-%02d" % ((x[0],) + x[1])),
                 (lambda x: []),
                 True,
                 _("club data"),
                 None,
                ),

    "arenaDetails.asp": ("arena details",
                         False, False,
                         [("FetchedDate", (lambda t: time.localtime(t)[0:4]))],
                         (lambda : [time.localtime(time.time())[0:4]]),
                         (lambda x: "%s-%04d-%02d-%02d-%02d" % ((x[0],) + x[1])),
                         (lambda x: []),
                         True,
                         _("arena details"),
                         None,
                         ),

    "matchesArchive.asp": ("match archive",
                           True, True,
                           [("Team.TeamID", None)],
                           (lambda : None),
                           (lambda x: "%s-t%d" % (x[0], x[1])),
                           (lambda x: [("FirstMatchDate", mkHTtimestring(x[1]))]),
                           False,
                           _("previous matches"),
                           None,
                           ),

    }

def create_name_table():
    """Takes the xml_known_xmls variable and takes out all HT names
    and Mattricks names of the xmls and makes the xml_name_table
    conversion table."""
    table = {}
    for filename, t in xml_known_xmls.items():
        table[t[0]] = filename
    return table

xml_name_table = create_name_table()

# Format of xml_known_data:
# {
# "data_section": {
#    "element.path": (content_description, content_conversion, content_display_conversion),
#    ...}
# ...}
xml_known_data = {
    #-----------------
    "unknown": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),

    },

    #-----------------
    "team details": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "User": None,
    "User.UserID": (_("User id"), int, None),
    "User.Language": None,
    "User.Language.LanguageID": (_("Language"), int, None), #TODO, add display conversion for language.
    "User.Language.LanguageName": (_("Language"), None, None),
    "User.HasSupporter": (_("Has supporter"), string2bool,
                          (lambda x: x and _("Yes") or _("No"))),
    "User.Loginname": (_("Username"), None, None),
    "User.Email": (_("Email address"), None, None),
    "User.Name": (_("Personal name"), None, None),
    "User.ICQ": (_("ICQ Number"), None, None),
    "User.SignupDate": (_("Date of signup"), string2time, mktimestring),
    "User.ActivationDate": (_("Date of activation"),
                            string2time, mktimestring),
    "User.LastLoginDate": (_("Date of last login"),
                           string2time, mktimestring),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.ShortTeamName": (_("Short team name"), None, None),
    "Team.Arena": None,
    "Team.Arena.ArenaID": (_("Arena id"), int, None),
    "Team.Arena.ArenaName": (_("Arena name"), None, None),
    "Team.League": None,
    "Team.League.LeagueID": (_("League id"), int, None),
    "Team.League.LeagueName": (_("League name"), None, None),
    "Team.Region": None,
    "Team.Region.RegionID": (_("Region id"), int, None),
    "Team.Region.RegionName": (_("Region name"), None, None),
    "Team.Trainer": None,
    "Team.Trainer.PlayerID": (_("Coach id"), int, None),
    "Team.Trainer.PlayerName": (_("Coach name"), None, None),
    "Team.HomePage": (_("Club homepage"), None, None),
    "Team.Cup": None,
    "Team.Cup.xml_attrs.Available": (None, string2bool,
                                     (lambda x: x and _("Yes") or _("No"))),
    "Team.Cup.StillInCup": (_("Team still in cup?"), string2bool,
                            (lambda x: x and _("Yes") or _("No"))),
    "Team.Cup.CupID": (_("Cup id"), int, None),
    "Team.Cup.CupName": (_("Cup name"), None, None),
    "Team.LeagueLevelUnit": None,
    "Team.LeagueLevelUnit.LeagueLevelUnitID": (_("Series id"), int, None),
    "Team.LeagueLevelUnit.LeagueLevelUnitName": (_("Series name"), None, None),
    "Team.LeagueLevelUnit.LeagueLevel": (_("Series level"), int, None),
    "Team.NumberOfVictories": (_("Consecutive victories"), int, None),
    "Team.NumberOfUndefeated": (_("Consecutive games without losing"), int, None),
    "Team.LogoURL": (_("URL of team logotype"), None, None),
    "Team.Fanclub": None,
    "Team.Fanclub.FanclubID": (_("Fanclub id"), int, None),
    "Team.Fanclub.FanclubName": (_("Fanclub name"), None, None),
    "Team.Guestbook": None,
    "Team.Guestbook.NumberOfGuestbookItems": (_("Number of items in guestbook"),
                                               int, None),
    "Team.PressAnnouncement": None,
    "Team.PressAnnouncement.Subject": (_("Subject of latest press announcement"),
                                       None, None),
    "Team.PressAnnouncement.Body": (_("Body of latest press announcement"),
                                    None, None),
    "Team.PressAnnouncement.SendDate": (_("Date when the latest press announcement was sent"),
                                        string2bool, mktimestring),
    "Team.Dress": (_("Team dress"),
                   (lambda d: (d[0], int(d[1:3]), int(d[3:5]), int(d[5:7]))),
                   None),
    "Team.DressAlternate": (_("Team dress"),
                            (lambda d: (d[0], int(d[1:3]), int(d[3:5]), int(d[5:7]))),
                            None),
    },

    #-----------------
    "league details": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "LeagueID": (_("League id"), int, None),
    "LeagueName": (_("League name"), None, None),
    "LeagueLevel": (_("Series level"), int, None),
    "MaxLevel": (_("Maximum league level"), int, None),
    "LeagueLevelUnitID": (_("Series id"), int, None),
    "LeagueLevelUnitName": (_("Series name"), None, None),
    "Team": None,
    "Team.xml_attrs.ID": (_("Team id"), int, None),
    "Team.xml_attrs.Index": (None, int, None),
    "Team.TeamID": (_("Team id"), int, None),
    "Team.Position": (_("Team position"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.Matches": (_("Matches played"), int, None),
    "Team.GoalsFor": (_("Goals for"), int, None),
    "Team.GoalsAgainst": (_("Goals against"), int, None),
    "Team.Points": (_("Team points"), int, None),
    },

    #-----------------
    "league fixtures": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "LeagueLevelUnitID": (_("Series id"), int, None),
    "LeagueLevelUnitName": (_("Series name"), None, None),
    "Season": (_("Season"), int, None),
    "Match": None,
    "Match.xml_attrs.ID": (_("Match id"), int, None),
    "Match.xml_attrs.Index": (None, int, None),
    "Match.MatchID": (_("Match id"), int, None),
    "Match.MatchRound": (_("Match round"), int, None),
    "Match.HomeTeam": None,
    "Match.HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "Match.HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "Match.AwayTeam": None,
    "Match.AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "Match.AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "Match.MatchDate": (_("Date of match"), string2time, mktimestring),
    "Match.HomeGoals": (_("Goals for home team"), int, None),
    "Match.AwayGoals": (_("Goals for away team"), int, None),
    },

    #-----------------
    "match list": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.MatchList": None,
    "Team.MatchList.Match": None,
    "Team.MatchList.Match.xml_attrs.Index": (None, int, None),
    "Team.MatchList.Match.MatchID": (_("Match id"), int, None),
    "Team.MatchList.Match.HomeTeam": None,
    "Team.MatchList.Match.HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "Team.MatchList.Match.HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "Team.MatchList.Match.AwayTeam": None,
    "Team.MatchList.Match.AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "Team.MatchList.Match.AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "Team.MatchList.Match.MatchDate": (_("Date of match"), string2time, mktimestring),
    "Team.MatchList.Match.MatchType": (_("Type of match"), int, dict2func(Terms.match_type)),
    "Team.MatchList.Match.HomeGoals": (_("Goals for home team"), int, None),
    "Team.MatchList.Match.AwayGoals": (_("Goals for away team"), int, None),
    "Team.MatchList.Match.Status": (_("Status of match"), None, dict2func(Terms.match_status)),
    "Team.MatchList.Match.OrdersGiven": (_("Orders given for match"), None, None),
    },

    #-----------------
    "match lineup": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "MatchID": (_("Match id"), int, None),
    "HomeTeam": None,
    "HomeTeam.xml_attrs.ID": (_("Home team id"), int, None),
    "HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "AwayTeam": None,
    "AwayTeam.xml_attrs.ID": (_("Away team id"), int, None),
    "AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "MatchType": (_("Match type"), int, dict2func(Terms.match_type)),
    "Arena": None,
    "Arena.xml_attrs.ID": (_("Arena id"), int, None),
    "Arena.ArenaID": (_("Arena id"), int, None),
    "Arena.ArenaName": (_("Arena name"), None, None),
    "MatchDate": (_("Date of match"), string2time, mktimestring),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.ExperienceLevel": (_("Experience level"), int, dict2func(Terms.formation_experience)),
    "Team.Lineup": None,
    "Team.Lineup.Player": None,
    "Team.Lineup.Player.xml_attrs.ID": (_("Player id"), int, None),
    "Team.Lineup.Player.xml_attrs.Index": (None, int, None),
    "Team.Lineup.Player.PlayerID": (_("Player id"), int, None),
    "Team.Lineup.Player.PlayerName": (_("Player name"), None, None),
    "Team.Lineup.Player.RoleID": (_("Role id"), int, dict2func(Terms.team_role)),
    "Team.Lineup.Player.RatingStars": (_("Rating"), float, None),
    "Team.Lineup.Player.PositionCode": (_("Position"), int, dict2func(Terms.team_role)),
    "Team.Lineup.Player.Behaviour": (_("Behaviour"), int, dict2func(Terms.individual_order)),
    },

    #-----------------
    "match orders": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "MatchID": (_("Match id"), int, None),
    "MatchData": None,
    "MatchData.xml_attrs.Available": (_("Match data available"), None, None),
    "MatchData.HomeTeam.xml_attrs.ID": (_("Home team id"), int, None),
    "MatchData.HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "MatchData.HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "MatchData.AwayTeam": None,
    "MatchData.AwayTeam.xml_attrs.ID": (_("Away team id"), int, None),
    "MatchData.AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "MatchData.AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "MatchData.MatchType": (_("Match type"), int, dict2func(Terms.match_type)),
    "MatchData.Arena": None,
    "MatchData.Arena.xml_attrs.ID": (_("Arena id"), int, None),
    "MatchData.Arena.ArenaID": (_("Arena id"), int, None),
    "MatchData.Arena.ArenaName": (_("Arena name"), None, None),
    "MatchData.MatchDate": (_("Date of match"), string2time, mktimestring),
    "MatchData.Attitude.xml_attrs.Available": (_("Team attitude"), None, None),
    "MatchData.Attitude": (_("Team attitude"), int, dict2func(Terms.team_attitude)),
    "MatchData.TacticType": (_("Team tactic"), int, dict2func(Terms.team_tactic)),
    "MatchData.Lineup": None,
    "MatchData.Lineup.Player": None,
    "MatchData.Lineup.Player.xml_attrs.ID": (_("Player id"), int, None),
    "MatchData.Lineup.Player.xml_attrs.Index": (None, int, None),
    "MatchData.Lineup.Player.PlayerID": (_("Player id"), int, None),
    "MatchData.Lineup.Player.PlayerName": (_("Player name"), None, None),
    "MatchData.Lineup.Player.RoleID": (_("Role id"), int, dict2func(Terms.team_role)),
    "MatchData.Lineup.Player.Behaviour": (_("Behaviour"), int, dict2func(Terms.individual_order)),
    },

    #-----------------
    "match details": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "Match": None,
    "Match.MatchID": (_("Match id"), int, None),
    "Match.MatchDate": (_("Date of match"), string2time, mktimestring),
    "Match.FinishedDate": (_("Date of end of match"), string2time, mktimestring),
    "Match.HomeTeam": None,
    "Match.HomeTeam.xml_attrs.ID": (_("Home team id"), int, None),
    "Match.HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "Match.HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "Match.HomeTeam.HomeGoals": (_("Home goals"), int, None),
    "Match.HomeTeam.Dress": (_("Team dress"),
                             (lambda d: (d[0], int(d[1:3]), int(d[3:5]), int(d[5:7]))),
                             None),
    "Match.HomeTeam.RatingMidfield": (_("Midfield rating"), int, dict2func(Terms.player_abilities)),
    "Match.HomeTeam.RatingRightDef": (_("Right defence rating"), int,
                                      dict2func(Terms.player_abilities)),
    "Match.HomeTeam.RatingMidDef": (_("Central defence rating"), int,
                                    dict2func(Terms.player_abilities)),
    "Match.HomeTeam.RatingLeftDef": (_("Left defence rating"), int,
                                     dict2func(Terms.player_abilities)),
    "Match.HomeTeam.RatingRightAtt": (_("Right attack rating"), int,
                                      dict2func(Terms.player_abilities)),
    "Match.HomeTeam.RatingMidAtt": (_("Central attack rating"), int,
                                    dict2func(Terms.player_abilities)),
    "Match.HomeTeam.RatingLeftAtt": (_("Left attack rating"), int, dict2func(Terms.player_abilities)),
    "Match.HomeTeam.TeamAttitude": (_("Team attitude"), int, dict2func(Terms.team_attitude)),
    "Match.HomeTeam.TacticType": (_("Team tactic"), int, dict2func(Terms.team_tactic)),
    "Match.HomeTeam.TacticSkill": (_("Tactic skill"), int, dict2func(Terms.player_abilities)),
    "Match.AwayTeam": None,
    "Match.AwayTeam.xml_attrs.ID": (_("Away team id"), int, None),
    "Match.AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "Match.AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "Match.AwayTeam.AwayGoals": (_("Away goals"), int, None),
    "Match.AwayTeam.Dress": (_("Team dress"),
                             (lambda d: (d[0], int(d[1:3]), int(d[3:5]), int(d[5:7]))),
                             None),
    "Match.AwayTeam.RatingMidfield": (_("Midfield rating"), int, dict2func(Terms.player_abilities)),
    "Match.AwayTeam.RatingRightDef": (_("Right defence rating"), int,
                                      dict2func(Terms.player_abilities)),
    "Match.AwayTeam.RatingMidDef": (_("Central defence rating"), int,
                                    dict2func(Terms.player_abilities)),
    "Match.AwayTeam.RatingLeftDef": (_("Left defence rating"), int,
                                     dict2func(Terms.player_abilities)),
    "Match.AwayTeam.RatingRightAtt": (_("Right attack rating"), int,
                                      dict2func(Terms.player_abilities)),
    "Match.AwayTeam.RatingMidAtt": (_("Central attack rating"), int,
                                    dict2func(Terms.player_abilities)),
    "Match.AwayTeam.RatingLeftAtt": (_("Left attack rating"), int, dict2func(Terms.player_abilities)),
    "Match.AwayTeam.TeamAttitude": (_("Team attitude"), int, dict2func(Terms.team_attitude)),
    "Match.AwayTeam.TacticType": (_("Team tactic"), int, dict2func(Terms.team_tactic)),
    "Match.AwayTeam.TacticSkill": (_("Tactic skill"), int, dict2func(Terms.player_abilities)),
    "Match.Arena": None,
    "Match.Arena.ArenaID": (_("Arena id"), int, None),
    "Match.Arena.ArenaName": (_("Arena name"), None, None),
    "Match.Arena.WeatherID": (_("Weather"), int, dict2func(Terms.weather)),
    "Match.Arena.SoldTotal": (_("Total sold tickets"), int, None),
    "Match.Arena.SoldTerraces": (_("Sold tickets of terraces"), int, None),
    "Match.Arena.SoldBasic": (_("Sold tickets of basic seating"), int, None),
    "Match.Arena.SoldRoof": (_("Sold tickets of roofed seating"), int, None),
    "Match.Arena.SoldVIP": (_("Sold tickets of seating in VIP boxes"), int, None),
    "Match.Scorers": None,
    "Match.Scorers.xml_attrs.NumberOfGoals": (_("Number of goals"), int, None),
    "Match.Scorers.Goal": None,
    "Match.Scorers.Goal.xml_attrs.Index": (None, int, None),
    "Match.Scorers.Goal.ScorerPlayerID": (_("Scoring player"), int, None),
    "Match.Scorers.Goal.ScorerPlayerName": (_("Scoring player"), None, None),
    "Match.Scorers.Goal.ScorerTeamID": (_("Scoring team"), int, None),
    "Match.Scorers.Goal.ScorerHomeGoals": (_("Home goals after scoring"), int, None),
    "Match.Scorers.Goal.ScorerAwayGoals": (_("Away goals after scoring"), int, None),
    "Match.Scorers.Goal.ScorerMinute": (_("Minute of score"), int, None),
    "Match.Bookings": None,
    "Match.Bookings.xml_attrs.NumberOfBookings": (_("Number of bookings"), int, None),
    "Match.Bookings.Booking": None,
    "Match.Bookings.Booking.xml_attrs.Index": (None, int, None),
    "Match.Bookings.Booking.BookingPlayerID": (_("Booked player"), int, None),
    "Match.Bookings.Booking.BookingPlayerName": (_("Booked player"), None, None),
    "Match.Bookings.Booking.BookingTeamID": (_("Booked team"), int, None),
    "Match.Bookings.Booking.BookingType": (_("Booking type"), int,
                                           (lambda b: (b == 1) and "Yellow" or "Red" )),
    "Match.Bookings.Booking.BookingMinute": (_("Minute of booking"), int, None),
    },

    #-----------------
    "HT live": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "ActionType": (_("Actiontype"), None, None),
    "MatchList": None,
    "MatchList.Match": None,
    "MatchList.Match.xml_attrs.Index": (None, int, None),
    "MatchList.Match.MatchID": (_("Match id"), int, None),
    "MatchList.Match.MatchDate": (_("Match date"), string2time, mktimestring),
    "MatchList.Match.HomeTeam": None,
    "MatchList.Match.HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "MatchList.Match.HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "MatchList.Match.AwayTeam": None,
    "MatchList.Match.AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "MatchList.Match.AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "MatchList.Match.HomeGoals": (_("Home goals"), int, None),
    "MatchList.Match.AwayGoals": (_("Away goals"), int, None),
    "MatchList.Match.EventList": None,
    "MatchList.Match.EventList.Event": None,
    "MatchList.Match.EventList.Event.xml_attrs.Index": (None, int, None),
    "MatchList.Match.EventList.Event.Minute": (_("Event minute"), int, None),
    "MatchList.Match.EventList.Event.EventKey": (_("Event key"),
                                                 (lambda x: [int(y) for y in x.split("_")]),
                                                 None),
    "MatchList.Match.EventList.Event.EventText": (_("Event text"), None, None),
    "MatchList.Match.EventList.Event.SubjectTeamID": (None, int, None),
    "MatchList.Match.EventList.Event.SubjectPlayerID": (None, int, None),
    "MatchList.Match.EventList.Event.ObjectPlayerID": (None, int, None),
    },

    #-----------------
    "world details": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "LanguageList": None,
    "LanguageList.Language": None,
    "LanguageList.Language.LanguageID": (_("Language"), int, None),
    "LanguageList.Language.LanguageName": (_("Language"), None, None),
    "LeagueList": None,
    "LeagueList.League": None,
    "LeagueList.League.xml_attrs.Index": (None, int, None),
    "LeagueList.League.LeagueID": (_("League"), int, None),
    "LeagueList.League.LeagueName": (_("League"), None, None),
    "LeagueList.League.Season": (_("Season"), int, None),
    "LeagueList.League.MatchRound": (_("Current match round"), int, None),
    "LeagueList.League.ShortName": (_("Short name of league"), None, None),
    "LeagueList.League.Continent": (_("Continent of league"), None, None),
    "LeagueList.League.EnglishName": (_("English name of league"), None, None),
    "LeagueList.League.Country": None,
    "LeagueList.League.Country.CountryID": (_("Country"), int, None),
    "LeagueList.League.Country.CountryName": (_("Country"), None, None),
    "LeagueList.League.Country.CurrencyName": (_("Currency"), None, None),
    "LeagueList.League.Country.CurrencyRate": (_("Currency rate"),
                                               (lambda x: float(x.replace(",", "."))), None),
    "LeagueList.League.Cup": None,
    "LeagueList.League.Cup.CupID": (_("Cup"), int, None),
    "LeagueList.League.Cup.CupName": (_("Cup"), None, None),
    "LeagueList.League.ActiveUsers": (_("Active users"), int, None),
    "LeagueList.League.WaitingUsers": (_("Waiting users"), int, None),
    "LeagueList.League.TrainingDate": (_("Training date"), string2time, mktimestring),
    "LeagueList.League.EconomyDate": (_("Economy date"), string2time, mktimestring),
    "LeagueList.League.CupMatchDate": (_("Cup match date"), string2time, mktimestring),
    "LeagueList.League.SeriesMatchDate": (_("Series match date"), string2time, mktimestring),
    "LeagueList.League.NumberOfLevels": (_("Number of levels"), int, None),
    },

    #-----------------
    "all players": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.PlayerList": None,
    "Team.PlayerList.Player": None,
    "Team.PlayerList.Player.PlayerID": (_("Player id"), int, None),
    "Team.PlayerList.Player.PlayerName": (_("Player name"),
                                          (lambda x: string.join(x.split(), " ")), None),
    "Team.PlayerList.Player.PlayerNumber": (_("Player number"), int, None),
    "Team.PlayerList.Player.Age": (_("Age"), int,
                              (lambda age: ngettext("%d year", "%d years", age) % age)),
    "Team.PlayerList.Player.MarketValue": (_("Assessed value"), int, CommonData.add_currency),
    "Team.PlayerList.Player.TSI": (_("Total Skill Index"), int, None),
    "Team.PlayerList.Player.PlayerForm": (_("Form"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.Statement": (_("Statement"), None, None),
    "Team.PlayerList.Player.Experience": (_("Experience"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.Leadership": (_("Leadership"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.Salary": (_("Salary"), int, CommonData.add_currency),
    "Team.PlayerList.Player.Agreeability": (_("Agreeability"), int, dict2func(Terms.agreeability)),
    "Team.PlayerList.Player.Aggressiveness": (_("Aggressiveness"), int,
                                              dict2func(Terms.aggressiveness)),
    "Team.PlayerList.Player.Honesty": (_("Honesty"), int, dict2func(Terms.honesty)),
    "Team.PlayerList.Player.LeagueGoals": (_("League goals"), int,
                                           (lambda goals: ngettext("%d goal",
                                                                   "%d goals", goals) % goals)),
    "Team.PlayerList.Player.CupGoals": (_("Cup goals"), int,
                                        (lambda goals: ngettext("%d goal",
                                                                "%d goals", goals) % goals)),
    "Team.PlayerList.Player.FriendliesGoals": (_("Goals in friendlies"), int,
                                               (lambda goals: ngettext("%d goal",
                                                                       "%d goals", goals) % goals)),
    "Team.PlayerList.Player.CareerGoals": (_("Career goals"), int,
                                           (lambda goals: ngettext("%d goal",
                                                                   "%d goals", goals) % goals)),
    "Team.PlayerList.Player.CareerHattricks": (_("Career hattricks"), int,
                                               (lambda goals: ngettext("%d hattrick",
                                                                       "%d hattricks",
                                                                       goals) % goals)),
    "Team.PlayerList.Player.Specialty": (_("Speciality"), int, dict2func(Terms.speciality)),
    "Team.PlayerList.Player.TransferListed": (_("Transferlisted"), string2bool,
                                              (lambda x: x and _("Yes") or _("No"))),
    "Team.PlayerList.Player.NationalTeamID": (_("National team"), int, (lambda cid: (cid != 0 and Countries.country_info[cid][1]) or "")),
    "Team.PlayerList.Player.Cards": (_("Bookings"), int, None),
    "Team.PlayerList.Player.InjuryLevel": (_("Injury"), int, dict2func(Terms.injury)),
    "Team.PlayerList.Player.StaminaSkill": (_("Stamina"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.KeeperSkill": (_("Goalkeeper"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.PlaymakerSkill": (_("Playmaking"), int,
                                              dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.ScorerSkill": (_("Scoring"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.PassingSkill": (_("Passing"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.WingerSkill": (_("Winger"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.DefenderSkill": (_("Defending"), int, dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.SetPiecesSkill": (_("Set pieces"), int,
                                              dict2func(Terms.player_abilities)),
    "Team.PlayerList.Player.OwningTeam": None,
    "Team.PlayerList.Player.OwningTeam.TeamID": (_("Owning team id"), int, None),
    "Team.PlayerList.Player.OwningTeam.TeamName": (_("Owning team name"), None, None),
    "Team.PlayerList.Player.OwningTeam.Leaguename": (_("Owning team league"), None, None),
    "Team.PlayerList.Player.TrainerData": None,
    "Team.PlayerList.Player.TrainerData.TrainerType": (_("Coach type"), int,
                                                       dict2func(Terms.coach_type)),
    "Team.PlayerList.Player.TrainerData.TrainerSkill": (_("Coaching ability"), int,
                                                   dict2func(Terms.coach_skills)),
    "Team.PlayerList.Player.CountryID": (_("Country"), int,
                                         (lambda cid: Countries.country_info[Countries.country[cid]][1])),
    "Team.PlayerList.Player.Caps": (_("National caps"), int, None),
    "Team.PlayerList.Player.CapsU20": (_("National U-20 caps"), int, None),
    },

    #-----------------
    "bookmarks": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "ActionType": (_("Type of information"), None, None),
    "UserBookmarks": None,
    "UserBookmarks.xml_attrs.Count": (None, int, None),
    "UserBookmarks.UserBookmark": None,
    "UserBookmarks.UserBookmark.xml_attrs.Index": (None, int, None),
    "UserBookmarks.UserBookmark.TeamID": (_("Team id"), int, None),
    "UserBookmarks.UserBookmark.TeamName": (_("Team name"), None, None),
    "UserBookmarks.UserBookmark.Loginname": (_("Username"), None, None),
    "UserBookmarks.UserBookmark.BookmarkID": (_("Bookmark id"), int, None),
    "PlayerBookmarks": None,
    "PlayerBookmarks.xml_attrs.Count": (None, int, None),
    "PlayerBookmarks.PlayerBookmark": None,
    "PlayerBookmarks.PlayerBookmark.xml_attrs.Index": (None, int, None),
    "PlayerBookmarks.PlayerBookmark.PlayerID": (_("Player id"), int, None),
    "PlayerBookmarks.PlayerBookmark.PlayerName": (_("Player name"), None, None),
    "PlayerBookmarks.PlayerBookmark.BookmarkID": (_("Bookmark id"), int, None),
    "MatchBookmarks": None,
    "MatchBookmarks.xml_attrs.Count": (None, int, None),
    "MatchBookmarks.MatchBookmark": None,
    "MatchBookmarks.MatchBookmark.xml_attrs.Index": (None, int, None),
    "MatchBookmarks.MatchBookmark.MatchID": (_("Match id"), int, None),
    "MatchBookmarks.MatchBookmark.MatchDate": (_("Match date"), None, None),
    "MatchBookmarks.MatchBookmark.HomeTeamName": (_("Home team name"), None, None),
    "MatchBookmarks.MatchBookmark.AwayTeamName": (_("Away team name"), None, None),
    "MatchBookmarks.MatchBookmark.BookmarkID": (_("Bookmark id"), int, None),
    "LeagueLevelUnitBookmarks": None,
    "LeagueLevelUnitBookmarks.xml_attrs.Count": (None, int, None),
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark": None,
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark.xml_attrs.Index": (None, int, None),
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark.LeagueLevelUnitID": (_("Series id"), int, None),
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark.LeagueLevelUnitName": (_("Series name"), None, None),
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark.LeagueID": (_("League id"), int, None),
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark.LeagueName": (_("League name"), None, None),
    "LeagueLevelUnitBookmarks.LeagueLevelUnitBookmark.BookmarkID": (_("Bookmark id"), int, None),
    "ConfBookmarks": None,
    "ConfBookmarks.xml_attrs.Count": (None, int, None),
    "ConfBookmarks.ConfBookmark": None,
    "ConfBookmarks.ConfBookmark.xml_attrs.Index": (None, int, None),
    "ConfBookmarks.ConfBookmark.UserID": (_("User id"), int, None),
    "ConfBookmarks.ConfBookmark.Loginname": (_("Username"), None, None),
    "ConfBookmarks.ConfBookmark.BookmarkID": (_("Bookmark id"), int, None),
    },

    #-----------------
    "transfer history": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "ActionType": (_("Type of information"), None, None),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.ActivatedDate": (_("activation date"), string2time, mktimestring),
    "Stats": None,
    "Stats.TotalSumOfBuys": (_("Total sum of purchases"), int, CommonData.add_currency),
    "Stats.TotalSumOfSales": (_("Total sum of sales"), int, CommonData.add_currency),
    "Stats.NumberOfBuys": (_("Number of purchases"), int, None),
    "Stats.NumberOfSales": (_("Number of sales"), int, None),
    "Transfers": None,
    "Transfers.xml_attrs.Count": (_("Number of transfers"), int, None),
    "Transfers.StartDate": (_("Date of first transfer in list"), string2time, mktimestring),
    "Transfers.EndDate": (_("Date of last transfer in list"), string2time, mktimestring),
    "Transfers.Player": None,
    "Transfers.Player.PlayerID": (_("Player id"), int, None),
    "Transfers.Player.PlayerName": (_("Player name"), None, None),
    "Transfers.Transfer": None,
    "Transfers.Transfer.xml_attrs.Index": (None, int, None),
    "Transfers.Transfer.TransferID": (_("Transfer id"), int, None),
    "Transfers.Transfer.Deadline": (_("Deadline date"), string2time, mkdatestring),
    "Transfers.Transfer.TransferType": (_("Transfer type"), None,
                                        (lambda x: (x == 'S' and _("Sale") or _("Purchase")))),
    "Transfers.Transfer.Player": None,
    "Transfers.Transfer.Player.PlayerID": (_("Player id"), int, None),
    "Transfers.Transfer.Player.PlayerName": (_("Player name"), None, None),
    "Transfers.Transfer.Buyer": None,
    "Transfers.Transfer.Buyer.BuyerTeamID": (_("Buyer id"), int, None),
    "Transfers.Transfer.Buyer.BuyerTeamName": (_("Buyer name"), None, None),
    "Transfers.Transfer.Seller": None,
    "Transfers.Transfer.Seller.SellerTeamID": (_("Seller id"), int, None),
    "Transfers.Transfer.Seller.SellerTeamName": (_("Seller name"), None, None),
    "Transfers.Transfer.Price": (_("Price"), int, CommonData.add_currency),
    "Transfers.Transfer.MarketValue": (_("Assessed value"), int, CommonData.add_currency),
    "Transfers.Transfer.TSI": (_("Total Skill Index"), int, None),
    },

    #-----------------
    "economy": {
        
    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "ActionType": (_("Type of information"), None, None),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.Cash": (None, int, CommonData.add_currency),
    "Team.ExpectedCash": (None, int, CommonData.add_currency),
    "Team.SponsorsPopularity": (None, int, dict2func(Terms.sponsors_and_supporters)),
    "Team.SupportersPopularity": (None, int, dict2func(Terms.sponsors_and_supporters)),
    "Team.FanClubSize": (None, int, None),
    "Team.IncomeSpectators": (None, int, CommonData.add_currency),
    "Team.IncomeSponsors": (None, int, CommonData.add_currency),
    "Team.IncomeFinancial": (None, int, CommonData.add_currency),
    "Team.IncomeTemporary": (None, int, CommonData.add_currency),
    "Team.IncomeSum": (None, int, CommonData.add_currency),
    "Team.CostsArena": (None, int, CommonData.add_currency),
    "Team.CostsPlayers": (None, int, CommonData.add_currency),
    "Team.CostsFinancial": (None, int, CommonData.add_currency),
    "Team.CostsTemporary": (None, int, CommonData.add_currency),
    "Team.CostsStaff": (None, int, CommonData.add_currency),
    "Team.CostsYouth": (None, int, CommonData.add_currency),
    "Team.CostsSum": (None, int, CommonData.add_currency),
    "Team.ExpectedWeeksTotal": (None, int, CommonData.add_currency),
    "Team.LastIncomeSpectators": (None, int, CommonData.add_currency),
    "Team.LastIncomeSponsors": (None, int, CommonData.add_currency),
    "Team.LastIncomeFinancial": (None, int, CommonData.add_currency),
    "Team.LastIncomeTemporary": (None, int, CommonData.add_currency),
    "Team.LastIncomeSum": (None, int, CommonData.add_currency),
    "Team.LastCostsArena": (None, int, CommonData.add_currency),
    "Team.LastCostsPlayers": (None, int, CommonData.add_currency),
    "Team.LastCostsFinancial": (None, int, CommonData.add_currency),
    "Team.LastCostsTemporary": (None, int, CommonData.add_currency),
    "Team.LastCostsStaff": (None, int, CommonData.add_currency),
    "Team.LastCostsYouth": (None, int, CommonData.add_currency),
    "Team.LastCostsSum": (None, int, CommonData.add_currency),
    "Team.LastWeeksTotal": (None, int, CommonData.add_currency),
    },

    #-----------------
    "training": {
        
    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "ActionType": (_("Type of information"), None, None),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.TrainingLevel": (None, int, (lambda x: str(x) + "%")),
    "Team.NewTrainingLevel": (None, int, (lambda x: str(x) + "%")),
    "Team.TrainingType": (None, int, dict2func(Terms.training_type)),
    "Team.Trainer": None,
    "Team.Trainer.TrainerID": (None, int, None),
    "Team.Trainer.TrainerName": (None, None, None),
    "Team.Trainer.ArrivalDate": (None, string2time, mktimestring),
    "Team.Morale": (None, int, dict2func(Terms.team_spirit)),
    "Team.SelfConfidence": (None, int, dict2func(Terms.team_self_confidence)),
    "Team.Experience433": (None, int, dict2func(Terms.formation_experience)),
    "Team.Experience451": (None, int, dict2func(Terms.formation_experience)),
    "Team.Experience352": (None, int, dict2func(Terms.formation_experience)),
    "Team.Experience532": (None, int, dict2func(Terms.formation_experience)),
    "Team.Experience343": (None, int, dict2func(Terms.formation_experience)),
    "Team.Experience541": (None, int, dict2func(Terms.formation_experience)),
    "League": None,
    "League.LeagueID": (None, int, None),
    "League.LeagueName": (None, None, None),
    "League.CreatedDate": (None, string2time, mktimestring),
    "League.TrainingStatList": None,
    "League.TrainingStatList.xml_attrs.Count": (None, int, None),
    "League.TrainingStatList.TrainingStat": None,
    "League.TrainingStatList.TrainingStat.xml_attrs.Index": (None, int, None),
    "League.TrainingStatList.TrainingStat.TrainingType": (None, int, dict2func(Terms.training_type)),
    "League.TrainingStatList.TrainingStat.NumberOfTeams": (None, int, None),
    "League.TrainingStatList.TrainingStat.FractionOfTeams": (None, float, (lambda x: "%.2f%%" % x)),
    },

    #-----------------
    "club": {
        
    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.Specialists": None,
    "Team.Specialists.KeeperTrainers": (_("Keeper trainers"), int, None),
    "Team.Specialists.AssistantTrainers": (_("Assistant trainers"), int, None),
    "Team.Specialists.Psychologists": (_("Psychologists"), int, None),
    "Team.Specialists.PressSpokesmen": (_("Press spokesmen"), int, None),
    "Team.Specialists.Economists": (_("Economists"), int, None),
    "Team.Specialists.Physiotherapists": (_("Physiotherapists"), int, None),
    "Team.Specialists.Doctors": (_("Doctors"), int, None),
    "Team.YouthSquad": None,
    "Team.YouthSquad.Investment": (_("Youth investment"), int, CommonData.add_currency),
    "Team.YouthSquad.HasPromoted": (_("Has promoted youth"), string2bool,
                                     (lambda x: x and _("Yes") or _("No"))),
    "Team.YouthSquad.YouthLevel": (_("Youth level"), int, dict2func(Terms.formation_experience)),
    },

    #-----------------
    "arena details": {
        
    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "Arena": None,
    "Arena.ArenaID": (_("Arena id"), int, None),
    "Arena.ArenaName": (_("Arena name"), None, None),
    "Arena.Team": None,
    "Arena.Team.TeamID": (_("Team id"), int, None),
    "Arena.Team.TeamName": (_("Team name"), None, None),
    "Arena.League": None,
    "Arena.League.LeagueID": (_("League id"), int, None),
    "Arena.League.LeageName": (_("League name"), None, None),
    "Arena.Region": None,
    "Arena.Region.RegionID": (_("Region id"), int, None),
    "Arena.Region.RegionName": (_("Region name"), None, None),
    "Arena.CurrentCapacity": None,
    "Arena.CurrentCapacity.RebuiltDate": (_("Rebuilt date"), string2time, mktimestring),
    "Arena.CurrentCapacity.Terraces": (_("Terraces"), int, None),
    "Arena.CurrentCapacity.Basic": (_("Basic seating"), int, None),
    "Arena.CurrentCapacity.Roof": (_("Roofed seating"), int, None),
    "Arena.CurrentCapacity.VIP": (_("Seating in VIP boxes"), int, None),
    "Arena.CurrentCapacity.Total": (_("Total capacity"), int, None),
    "Arena.ExpandedCapacity": None,
    "Arena.ExpandedCapacity.xml_attr.Available": (_("Under expansion"), None, None),
    "Arena.ExpandedCapacity.ExpansionDate": (_("Expansion date"), string2time, mktimestring),
    "Arena.ExpandedCapacity.Terraces": (_("Expanded terraces"), int, None),
    "Arena.ExpandedCapacity.Basic": (_("Expanded basic seating"), int, None),
    "Arena.ExpandedCapacity.Roof": (_("Expanded roofed seating"), int, None),
    "Arena.ExpandedCapacity.VIP": (_("Expanded seating in VIP boxes"), int, None),    
    },

    #-----------------
    "match archive": {

    "UserID": (_("User id"), int, None),
    "Version": (_("File version"), None, None),
    "FetchedDate": (_("Date of fetch"), string2time, mktimestring),
    "Team": None,
    "Team.TeamID": (_("Team id"), int, None),
    "Team.TeamName": (_("Team name"), None, None),
    "Team.FirstMatchDate": (_("Date of first match in list"), string2time, mktimestring),
    "Team.LastMatchDate": (_("Date of last match in list"), string2time, mktimestring),
    "Team.MatchList": None,
    "Team.MatchList.xml_attrs.Count": (None, int, None),
    "Team.MatchList.Match": None,
    "Team.MatchList.Match.xml_attrs.Index": (None, int, None),
    "Team.MatchList.Match.MatchID": (_("Match id"), int, None),
    "Team.MatchList.Match.HomeTeam": None,
    "Team.MatchList.Match.HomeTeam.HomeTeamID": (_("Home team id"), int, None),
    "Team.MatchList.Match.HomeTeam.HomeTeamName": (_("Home team name"), None, None),
    "Team.MatchList.Match.AwayTeam": None,
    "Team.MatchList.Match.AwayTeam.AwayTeamID": (_("Away team id"), int, None),
    "Team.MatchList.Match.AwayTeam.AwayTeamName": (_("Away team name"), None, None),
    "Team.MatchList.Match.MatchDate": (_("Date of match"), string2time, mktimestring),
    "Team.MatchList.Match.MatchType": (_("Type of match"), int, dict2func(Terms.match_type)),
    "Team.MatchList.Match.HomeGoals": (_("Goals for home team"), int, None),
    "Team.MatchList.Match.AwayGoals": (_("Goals for away team"), int, None),
    },

}
